MODULE trcbio_medusa
   !!======================================================================
   !!                         ***  MODULE trcbio  ***
   !! TOP :   MEDUSA
   !!======================================================================
   !! History :
   !!  -   !  1999-07  (M. Levy)              original code
   !!  -   !  2000-12  (E. Kestenare)         assign parameters to name individual tracers
   !!  -   !  2001-03  (M. Levy)              LNO3 + dia2d 
   !! 2.0  !  2007-12  (C. Deltel, G. Madec)  F90
   !!  -   !  2008-08  (K. Popova)            adaptation for MEDUSA
   !!  -   !  2008-11  (A. Yool)              continuing adaptation for MEDUSA
   !!  -   !  2010-03  (A. Yool)              updated for branch inclusion
   !!  -   !  2011-08  (A. Yool)              updated for ROAM (see below)
   !!  -   !  2013-03  (A. Yool)              updated for iMARNET
   !!  -   !  2013-05  (A. Yool)              updated for v3.5
   !!  -   !  2014-08  (A. Yool, J. Palm)     Add DMS module for UKESM1 model
   !!  -   !  2015-06  (A. Yool)              Update to include MOCSY
   !!  -   !  2015-07  (A. Yool)              Update for rolling averages
   !!  -   !  2015-10  (J. Palm)              Update for diag outputs through iom_use  
   !!  -   !  2016-11  (A. Yool)              Updated diags for CMIP6
   !!----------------------------------------------------------------------
   !!
#if defined key_roam
   !!----------------------------------------------------------------------
   !! Updates for the ROAM project include:
   !!   - addition of DIC, alkalinity, detrital carbon and oxygen tracers
   !!   - addition of air-sea fluxes of CO2 and oxygen
   !!   - periodic (monthly) calculation of full 3D carbonate chemistry 
   !!   - detrital C:N ratio now free to evolve dynamically
   !!   - benthic storage pools
   !! 
   !! Opportunity also taken to add functionality:
   !!   - switch for Liebig Law (= most-limiting) nutrient uptake
   !!   - switch for accelerated seafloor detritus remineralisation
   !!   - switch for fast -> slow detritus transfer at seafloor
   !!   - switch for ballast vs. Martin vs. Henson fast detritus remin.
   !!   - per GMD referee remarks, xfdfrac3 introduced for grazed PDS
   !!----------------------------------------------------------------------
#endif
   !!
#if defined key_mocsy
   !!----------------------------------------------------------------------
   !! Updates with the addition of MOCSY include:
   !!   - option to use PML or MOCSY carbonate chemistry (the latter is 
   !!     preferred)
   !!   - central calculation of gas transfer velocity, f_kw660; previously
   !!     this was done separately for CO2 and O2 with predictable results
   !!   - distribution of f_kw660 to both PML and MOCSY CO2 air-sea flux 
   !!     calculations and to those for O2 air-sea flux
   !!   - extra diagnostics included for MOCSY
   !!----------------------------------------------------------------------
#endif
   !!
#if defined key_medusa
   !!----------------------------------------------------------------------
   !!                                        MEDUSA bio-model
   !!----------------------------------------------------------------------
   !!   trc_bio_medusa        :  
   !!----------------------------------------------------------------------
      USE oce_trc
      USE trc
      USE sms_medusa
      USE lbclnk
      USE prtctl_trc      ! Print control for debugging
      USE trcsed_medusa
      USE sbc_oce         ! surface forcing
      USE sbcrnf          ! surface boundary condition: runoff variables
      USE in_out_manager  ! I/O manager
# if defined key_iomput
      USE iom
      USE trcnam_medusa         ! JPALM 13-11-2015 -- if iom_use for diag
      !!USE trc_nam_iom_medusa  ! JPALM 13-11-2015 -- if iom_use for diag
# endif
# if defined key_roam
      USE gastransfer
#  if defined key_mocsy
      USE mocsy_wrapper
#  else
      USE trcco2_medusa
#  endif
      USE trcoxy_medusa
      !! Jpalm (08/08/2014)
      USE trcdms_medusa
# endif
      !! AXY (18/01/12): brought in for benthic timestepping
      USE trcnam_trp      ! AXY (24/05/2013)
      USE trdmxl_trc
      USE trdtrc_oce  ! AXY (24/05/2013)

      !! AXY (30/01/14): necessary to find NaNs on HECTOR
      USE, INTRINSIC :: ieee_arithmetic 

      !! JPALM (27-06-2016): add lk_oasis for CO2 and DMS coupling with atm
      USE sbc_oce,                    ONLY: lk_oasis
      USE oce,                        ONLY: CO2Flux_out_cpl, DMS_out_cpl,   &
                                            PCO2a_in_cpl
      USE bio_medusa_mod
      USE bio_medusa_init_mod,        ONLY: bio_medusa_init
      USE carb_chem_mod,              ONLY: carb_chem
      USE air_sea_mod,                ONLY: air_sea
      USE plankton_mod,               ONLY: plankton
      USE bio_medusa_diag_slice_mod,  ONLY: bio_medusa_diag_slice
      USE bio_medusa_fin_mod,         ONLY: bio_medusa_fin

      IMPLICIT NONE
      PRIVATE
      
      PUBLIC   trc_bio_medusa    ! called in trcsms_medusa.F90

   !!* Substitution
#  include "domzgr_substitute.h90"
   !!----------------------------------------------------------------------
   !! NEMO/TOP 2.0 , LOCEAN-IPSL (2007) 
   !! $Id$
   !! Software governed by the CeCILL licence (modipsl/doc/NEMO_CeCILL.txt)
   !!----------------------------------------------------------------------

CONTAINS

   SUBROUTINE trc_bio_medusa( kt )
      !!---------------------------------------------------------------------
      !!                     ***  ROUTINE trc_bio  ***
      !!
      !! ** Purpose :   compute the now trend due to biogeochemical processes
      !!              and add it to the general trend of passive tracers equations
      !!
      !! ** Method  :   each now biological flux is calculated in function of now
      !!              concentrations of tracers.
      !!              depending on the tracer, these fluxes are sources or sinks.
      !!              the total of the sources and sinks for each tracer
      !!              is added to the general trend.
      !!        
      !!                      tra = tra + zf...tra - zftra...
      !!                                     |         |
      !!                                     |         |
      !!                                  source      sink
      !!        
      !!              IF 'key_trc_diabio' defined , the biogeochemical trends
      !!              for passive tracers are saved for futher diagnostics.
      !!---------------------------------------------------------------------
      !!
      !!
      !!----------------------------------------------------------------------            
      !! Variable conventions
      !!----------------------------------------------------------------------
      !!
      !! names: z*** - state variable 
      !!        f*** - function (or temporary variable used in part of a function)
      !!        x*** - parameter
      !!        b*** - right-hand part (sources and sinks)
      !!        i*** - integer variable (usually used in yes/no flags)
      !!
      !! time (integer timestep)
      INTEGER, INTENT( in ) ::    kt
      !!
      !! spatial array indices
      INTEGER  ::    ji,jj,jk,jn
      !!
      !! AXY (27/07/10): add in indices for depth horizons (for sinking flux
      !!                 and seafloor iron inputs)
      !! INTEGER  ::    i0100, i0200, i0500, i1000, i1100
      !!
      !! model state variables
!      REAL(wp), DIMENSION(jpi,jpj) ::    zchn,zchd,zphn,zphd,zpds,zzmi
!      REAL(wp), DIMENSION(jpi,jpj) ::    zzme,zdet,zdtc,zdin,zsil,zfer
! zage doesn't seem to be used - marc 19/4/17
!      REAL(wp) ::    zage
!# if defined key_roam
!      REAL(wp), DIMENSION(jpi,jpj) ::    zdic, zalk, zoxy
!      REAL(wp), DIMENSION(jpi,jpj) ::    ztmp, zsal
!# endif
!# if defined key_mocsy
!      REAL(wp), DIMENSION(jpi,jpj) ::    zpho
!# endif
      !!
      !! integrated source and sink terms
      REAL(wp) ::    b0
      !! AXY (23/08/13): changed from individual variables for each flux to
      !!                 an array that holds all fluxes
      REAL(wp), DIMENSION(jpi,jpj,jp_medusa) ::    btra
      !!
      !! primary production and chl related quantities      
!      REAL(wp), DIMENSION(jpi,jpj) ::    fthetan,faln,fchn1,fchn,fjln,fprn,frn
!      REAL(wp), DIMENSION(jpi,jpj) ::    fthetad,fald,fchd1,fchd,fjld,fprd,frd
      !! AXY (23/11/16): add in light-only limitation term (normalised 0-1 range)
!      REAL(wp), DIMENSION(jpi,jpj) ::    fjlim_pn, fjlim_pd
      !! AXY (03/02/11): add in Liebig terms
!      REAL(wp), DIMENSION(jpi,jpj) ::    fpnlim, fpdlim
      !! AXY (16/07/09): add in Eppley curve functionality
!      REAL(wp), DIMENSION(jpi,jpj) ::    fun_T,xvpnT,xvpdT
      INTEGER  ::    ieppley
      !! AXY (16/05/11): per Katya's prompting, add in new T-dependence
      !!                 for phytoplankton growth only (i.e. no change
      !!                 for remineralisation)
!      REAL(wp), DIMENSION(jpi,jpj) ::    fun_Q10
      !! AXY (01/03/10): add in mixed layer PP diagnostics
!      REAL(wp), DIMENSION(jpi,jpj) ::    fprn_ml,fprd_ml
      !!
      !! nutrient limiting factors
!      REAL(wp), DIMENSION(jpi,jpj) ::    fnln,ffln2            !! N and Fe
!      REAL(wp), DIMENSION(jpi,jpj) ::    fnld,ffld,fsld,fsld2 !! N, Fe and Si
      !!
      !! silicon cycle
!      REAL(wp), DIMENSION(jpi,jpj) ::    fsin,fnsi,fprds,fsdiss
      REAL(wp)                     ::    fsin1,fnsi1,fnsi2
      !!
      !! iron cycle; includes parameters for Parekh et al. (2005) iron scheme
      REAL(wp), DIMENSION(jpi,jpj) ::    ffetop,ffebot,ffescav
      REAL(wp) ::    xLgF, xFeT, xFeF, xFeL         !! state variables for iron-ligand system
!      REAL(wp), DIMENSION(jpi,jpj) ::  xFree        !! state variables for iron-ligand system
      REAL(wp) ::    xb_coef_tmp, xb2M4ac           !! iron-ligand parameters
      REAL(wp) ::    xmaxFeF,fdeltaFe               !! max Fe' parameters
      !!
      !! local parameters for Moore et al. (2004) alternative scavenging scheme
      REAL(wp) ::    fbase_scav,fscal_sink,fscal_part,fscal_scav
      !!
      !! local parameters for Moore et al. (2008) alternative scavenging scheme
      REAL(wp) ::    fscal_csink,fscal_sisink,fscal_casink
      !!
      !! local parameters for Galbraith et al. (2010) alternative scavenging scheme
      REAL(wp) ::    xCscav1, xCscav2, xk_org, xORGscav  !! organic portion of scavenging
      REAL(wp) ::    xk_inorg, xINORGscav                !! inorganic portion of scavenging
      !!
      !! microzooplankton grazing
!      REAL(wp), DIMENSION(jpi,jpj) ::    fmi1,fmi,fgmipn,fgmid,fgmidc
!      REAL(wp), DIMENSION(jpi,jpj) ::    finmi,ficmi,fstarmi,fmith,fmigrow,fmiexcr,fmiresp
      !!
      !! mesozooplankton grazing
!      REAL(wp), DIMENSION(jpi,jpj) ::    fme1,fme,fgmepn,fgmepd,fgmepds,fgmezmi,fgmed,fgmedc
!      REAL(wp), DIMENSION(jpi,jpj) ::    finme,ficme,fstarme,fmeth,fmegrow,fmeexcr,fmeresp
      !!
      !! mortality/Remineralisation (defunct parameter "fz" removed)
!      REAL(wp), DIMENSION(jpi,jpj) ::    fdpn,fdpd,fdpds,fdzmi,fdzme,fdd
# if defined key_roam
      REAL(wp), DIMENSION(jpi,jpj) ::    fddc
# endif
!      REAL(wp), DIMENSION(jpi,jpj) ::    fdpn2,fdpd2,fdpds2,fdzmi2,fdzme2
      REAL(wp), DIMENSION(jpi,jpj) ::    fslown, fslowc
!      REAL(wp), DIMENSION(jpi,jpj) ::    fslownflux, fslowcflux
      REAL(wp), DIMENSION(jpi,jpj) ::    fregen,fregensi
!      REAL(wp), DIMENSION(jpi,jpj) ::    fregenfast,fregenfastsi
# if defined key_roam
!! Doesn't look like this is used - marc 10/4/17
!!      REAL(wp), DIMENSION(jpi,jpj) ::    fregenc
!      REAL(wp), DIMENSION(jpi,jpj) ::    fregenfastc
# endif
      !!
      !! particle flux
!      REAL(WP), DIMENSION(jpi,jpj) ::    fdep1,fcaco3
      REAL(WP), DIMENSION(jpi,jpj) ::    ftempn,ftempsi,ftempfe,ftempc,ftempca
      REAL(wp), DIMENSION(jpi,jpj) ::    freminn,freminsi,freminfe,freminc,freminca
!      REAL(wp), DIMENSION(jpi,jpj) ::    ffastn,ffastsi,ffastfe,ffastc,ffastca
      REAL(wp), DIMENSION(jpi,jpj) ::    fprotf
!      REAL(wp), DIMENSION(jpi,jpj) ::    fsedn,fsedsi,fsedfe,fsedc,fsedca
!      REAL(wp), DIMENSION(jpi,jpj) ::    fccd
      REAL(wp), DIMENSION(jpi,jpj) ::    fccd_dep
      !!
      !! AXY (06/07/11): alternative fast detritus schemes
      REAL(wp) ::    fb_val, fl_sst
      !!
      !! AXY (08/07/11): fate of fast detritus reaching the seafloor
! I don't think ffast2slowfe is used - marc 10/4/17
!      REAL(wp), DIMENSION(jpi,jpj) ::    ffast2slown,ffast2slowfe,ffast2slowc
      REAL(wp), DIMENSION(jpi,jpj) ::    ffast2slown,ffast2slowc
      !!
      !! conservation law
      REAL(wp) ::    fnit0,fsil0,ffer0 
# if defined key_roam
      REAL(wp) ::    fcar0,falk0,foxy0 
# endif      
      !! 
      !! temporary variables
      REAL(wp) ::    fq0,fq1,fq2,fq3,fq4,fq5,fq6,fq7,fq8,fq9
      !!
      !! water column nutrient and flux integrals
!      REAL(wp), DIMENSION(jpi,jpj) ::    ftot_n,ftot_si,ftot_fe
!      REAL(wp), DIMENSION(jpi,jpj) ::    fflx_n,fflx_si,fflx_fe
!      REAL(wp), DIMENSION(jpi,jpj) ::    fifd_n,fifd_si,fifd_fe
!      REAL(wp), DIMENSION(jpi,jpj) ::    fofd_n,fofd_si,fofd_fe
# if defined key_roam
!      REAL(wp), DIMENSION(jpi,jpj) ::    ftot_c,ftot_a,ftot_o2
!      REAL(wp), DIMENSION(jpi,jpj) ::    fflx_c,fflx_a,fflx_o2
!      REAL(wp), DIMENSION(jpi,jpj) ::    fifd_c,fifd_a,fifd_o2
!      REAL(wp), DIMENSION(jpi,jpj) ::    fofd_c,fofd_a,fofd_o2
# endif
      !!
      !! zooplankton grazing integrals
!      REAL(wp), DIMENSION(jpi,jpj) ::    fzmi_i,fzmi_o,fzme_i,fzme_o
      !!
      !! limitation term temporary variables
!      REAL(wp), DIMENSION(jpi,jpj) ::    ftot_pn,ftot_pd
!      REAL(wp), DIMENSION(jpi,jpj) ::    ftot_zmi,ftot_zme,ftot_det,ftot_dtc
      !! use ballast scheme (1) or simple exponential scheme (0; a conservation test)
      INTEGER  ::    iball
      !! use biological fluxes (1) or not (0)
      INTEGER  ::    ibio_switch
      !!
      !! diagnose fluxes (should only be used in 1D runs)
!      INTEGER  ::    idf, idfval
      !!
      !! nitrogen and silicon production and consumption
      REAL(wp) ::    fn_prod, fn_cons, fs_prod, fs_cons
!      REAL(wp), DIMENSION(jpi,jpj) ::    fnit_prod, fnit_cons, fsil_prod, fsil_cons
# if defined key_roam
      !!
      !! flags to help with calculating the position of the CCD
! Moved into carb_chem.F90 - marc 20/4/17
!      INTEGER, DIMENSION(jpi,jpj) ::     i2_omcal,i2_omarg
      !!
      !! AXY (24/11/16): add xCO2 variable for atmosphere (what we actually have)
!      REAL(wp)                     ::    f_xco2a
!      REAL(wp), DIMENSION(jpi,jpj) ::    f_ph, f_pco2w, f_h2co3, f_hco3, f_co3, f_co2flux
!      REAL(wp), DIMENSION(jpi,jpj) ::    f_TDIC, f_TALK, f_dcf, f_henry
!      REAL(wp), DIMENSION(jpi,jpj) ::    f_pp0
!      REAL(wp), DIMENSION(jpi,jpj) ::    f_kw660, f_o2flux, f_o2sat
      REAL(wp)                     ::    f_o2sat3
!      REAL(wp), DIMENSION(jpi,jpj) ::    f_omcal, f_omarg
      !!
      !! AXY (23/06/15): additional diagnostics for MOCSY and oxygen
!      REAL(wp), DIMENSION(jpi,jpj) ::    f_fco2w, f_BetaD, f_rhosw, f_opres, f_insitut, f_pco2atm, f_fco2atm
!      REAL(wp), DIMENSION(jpi,jpj) ::    f_schmidtco2, f_kwco2, f_K0, f_co2starair, f_dpco2, f_kwo2
      !! jpalm 14-07-2016: convert CO2flux diag from mmol/m2/d to kg/m2/s
      REAL, PARAMETER :: weight_CO2_mol = 44.0095  !! g / mol
      REAL, PARAMETER :: secs_in_day    = 86400.0  !! s / d
      REAL, PARAMETER :: CO2flux_conv   = (1.e-6 * weight_CO2_mol) / secs_in_day

      !!
!      INTEGER, DIMENSION(jpi,jpj)  ::    iters
      REAL(wp) ::    f_year
      INTEGER  ::    i_year
      INTEGER  ::    iyr1, iyr2
      !!
      !! carbon, alkalinity production and consumption
      REAL(wp) ::    fc_prod, fc_cons, fa_prod, fa_cons
!      REAL(wp), DIMENSION(jpi,jpj) ::    fcomm_resp
!      REAL(wp), DIMENSION(jpi,jpj) ::    fcar_prod, fcar_cons
      !!
      !! oxygen production and consumption (and non-consumption)
      REAL(wp), DIMENSION(jpi,jpj) ::    fo2_prod, fo2_cons, fo2_ncons, fo2_ccons
!      REAL(wp), DIMENSION(jpi,jpj) ::    foxy_prod, foxy_cons, foxy_anox
      !! Jpalm (11-08-2014)
      !! add DMS in MEDUSA for UKESM1 model
!      REAL(wp), DIMENSION(jpi,jpj) ::    dms_surf
      !! AXY (13/03/15): add in other DMS calculations
!      REAL(wp), DIMENSION(jpi,jpj) ::    dms_andr, dms_simo, dms_aran, dms_hall

# endif
      !! 
      !! benthic fluxes
!      INTEGER  ::    ibenthic
!      REAL(wp), DIMENSION(jpi,jpj) :: f_sbenin_n, f_sbenin_fe,              f_sbenin_c
!      REAL(wp), DIMENSION(jpi,jpj) :: f_fbenin_n, f_fbenin_fe, f_fbenin_si, f_fbenin_c, f_fbenin_ca
!      REAL(wp), DIMENSION(jpi,jpj) :: f_benout_n, f_benout_fe, f_benout_si, f_benout_c, f_benout_ca
      REAL(wp) ::    zfact
      !! 
      !! benthic fluxes of CaCO3 that shouldn't happen because of lysocline
!      REAL(wp), DIMENSION(jpi,jpj) :: f_benout_lyso_ca
      !!
      !! riverine fluxes
!      REAL(wp), DIMENSION(jpi,jpj) :: f_runoff, f_riv_n, f_riv_si, f_riv_c, f_riv_alk
      !! AXY (19/07/12): variables for local riverine fluxes to handle inputs below surface
      REAL(wp), DIMENSION(jpi,jpj) :: f_riv_loc_n, f_riv_loc_si, f_riv_loc_c, f_riv_loc_alk
      !!---------------------------------------------------------------------

# if defined key_debug_medusa
      IF (lwp) write (numout,*) 'trc_bio_medusa: variables defined'
      CALL flush(numout)
# endif 

      !! AXY (20/11/14): alter this to report on first MEDUSA call
      !! IF( kt == nit000 ) THEN
      IF( kt == nittrc000 ) THEN
         IF(lwp) WRITE(numout,*)
         IF(lwp) WRITE(numout,*) ' trc_bio: MEDUSA bio-model'
         IF(lwp) WRITE(numout,*) ' ~~~~~~~'
	 IF(lwp) WRITE(numout,*) ' kt =',kt
      ENDIF

      !! AXY (13/01/12): is benthic model properly interactive? 0 = no, 1 = yes
      ibenthic = 1

      !! not sure what this is for; it's not used anywhere; commenting out
      !! fbodn(:,:) = 0.e0   

      !!
      IF( ln_diatrc ) THEN
         !! blank 2D diagnostic array
         trc2d(:,:,:) = 0.e0
         !!
         !! blank 3D diagnostic array
         trc3d(:,:,:,:) = 0.e0
      ENDIF

      !!----------------------------------------------------------------------
      !! b0 is present for debugging purposes; using b0 = 0 sets the tendency
      !! terms of all biological equations to 0.
      !!----------------------------------------------------------------------
      !!
      !! AXY (03/09/14): probably not the smartest move ever, but it'll fit
      !!                 the bill for now; another item on the things-to-sort-
      !!		 out-in-the-future list ...
# if defined key_kill_medusa
      b0 = 0.
# else
      b0 = 1.
# endif
      !!----------------------------------------------------------------------
      !! fast detritus ballast scheme (0 = no; 1 = yes)
      !! alternative to ballast scheme is same scheme but with no ballast
      !! protection (not dissimilar to Martin et al., 1987)
      !!----------------------------------------------------------------------
      !!
      iball = 1

      !!----------------------------------------------------------------------
      !! full flux diagnostics (0 = no; 1 = yes); appear in ocean.output
      !! these should *only* be used in 1D since they give comprehensive
      !! output for ecological functions in the model; primarily used in
      !! debugging
      !!----------------------------------------------------------------------
      !!
      idf    = 0
      !!
      !! timer mechanism
      if (kt/120*120.eq.kt) then
         idfval = 1
      else
         idfval = 0
      endif

      !!----------------------------------------------------------------------
      !! Initialise arrays to zero and set up arrays for diagnostics
      !!----------------------------------------------------------------------
! tmp - marc
      write(numout,*) 'bbb13. before call to bio_medusa_init,kt=',kt
      flush(numout)
!
      CALL bio_medusa_init( kt )
! tmp - marc
      write(numout,*) 'bbb14. after call to bio_medusa_init,kt=',kt
      flush(numout)
!
       !!
# if defined key_axy_nancheck
       DO jn = 1,jptra
          !! fq0 = MINVAL(trn(:,:,:,jn))
          !! fq1 = MAXVAL(trn(:,:,:,jn))
          fq2 = SUM(trn(:,:,:,jn))
          !! if (lwp) write (numout,'(a,2i6,3(1x,1pe15.5))') 'NAN-CHECK', &
          !! &        kt, jn, fq0, fq1, fq2
          !! AXY (30/01/14): much to our surprise, the next line doesn't work on HECTOR
          !!                 and has been replaced here with a specialist routine
          !! if (fq2 /= fq2 ) then
          if ( ieee_is_nan( fq2 ) ) then
             !! there's a NaN here
             if (lwp) write(numout,*) 'NAN detected in field', jn, 'at time', kt, 'at position:'
             DO jk = 1,jpk
                DO jj = 1,jpj
                   DO ji = 1,jpi
                      !! AXY (30/01/14): "isnan" problem on HECTOR
                      !! if (trn(ji,jj,jk,jn) /= trn(ji,jj,jk,jn)) then
                      if ( ieee_is_nan( trn(ji,jj,jk,jn) ) ) then
                         if (lwp) write (numout,'(a,1pe12.2,4i6)') 'NAN-CHECK', &
                         &        tmask(ji,jj,jk), ji, jj, jk, jn
                      endif
                   enddo
                enddo
             enddo
             CALL ctl_stop( 'trcbio_medusa, NAN in incoming tracer field' )
          endif
       ENDDO
       CALL flush(numout)
# endif

# if defined key_debug_medusa
      IF (lwp) write (numout,*) 'trc_bio_medusa: variables initialised and checked'
      CALL flush(numout)
# endif 

# if defined key_roam
      !!----------------------------------------------------------------------
      !! calculate atmospheric pCO2
      !!----------------------------------------------------------------------
      !!
      !! what's atmospheric pCO2 doing? (data start in 1859)
      iyr1  = nyear - 1859 + 1
      iyr2  = iyr1 + 1
      if (iyr1 .le. 1) then
         !! before 1860
         f_xco2a(:,:) = hist_pco2(1)
      elseif (iyr2 .ge. 242) then
         !! after 2099
         f_xco2a(:,:) = hist_pco2(242)
      else
         !! just right
         fq0 = hist_pco2(iyr1)
         fq1 = hist_pco2(iyr2)
         fq2 = real(nsec_day) / (60.0 * 60.0 * 24.0)
         !! AXY (14/06/12): tweaked to make more sense (and be correct)
#  if defined key_bs_axy_yrlen
         fq3 = (real(nday_year) - 1.0 + fq2) / 360.0  !! bugfix: for 360d year with HadGEM2-ES forcing
#  else
         fq3 = (real(nday_year) - 1.0 + fq2) / 365.0  !! original use of 365 days (not accounting for leap year or 360d year)
#  endif
         fq4 = (fq0 * (1.0 - fq3)) + (fq1 * fq3)
         f_xco2a(:,:) = fq4
      endif
#  if defined key_axy_pi_co2
      f_xco2a(:,:) = 284.725          !! OCMIP pre-industrial pCO2
#  endif
      !! IF(lwp) WRITE(numout,*) ' MEDUSA nyear     =', nyear
      !! IF(lwp) WRITE(numout,*) ' MEDUSA nsec_day  =', real(nsec_day)
      !! IF(lwp) WRITE(numout,*) ' MEDUSA nday_year =', real(nday_year)
      !! AXY (29/01/14): remove surplus diagnostics
      !! IF(lwp) WRITE(numout,*) ' MEDUSA fq0       =', fq0
      !! IF(lwp) WRITE(numout,*) ' MEDUSA fq1       =', fq1
      !! IF(lwp) WRITE(numout,*) ' MEDUSA fq2       =', fq2
      !! IF(lwp) WRITE(numout,*) ' MEDUSA fq3       =', fq3
      IF(lwp) WRITE(numout,*) ' MEDUSA atm pCO2  =', f_xco2a(1,1)
# endif

# if defined key_debug_medusa
      IF (lwp) write (numout,*) 'trc_bio_medusa: ready for carbonate chemistry'
      IF (lwp) write (numout,*) 'trc_bio_medusa: kt = ', kt
      IF (lwp) write (numout,*) 'trc_bio_medusa: nittrc000 = ', nittrc000
      CALL flush(numout)
# endif 

# if defined key_roam
      !! AXY (20/11/14): alter to call on first MEDUSA timestep and then every
      !!                 month (this is hardwired as 960 timesteps but should
      !!                 be calculated and done properly
      !! IF( kt == nit000 .or. mod(kt,1920) == 0 ) THEN
      !! IF( kt == nittrc000 .or. mod(kt,960) == 0 ) THEN 
      !!=============================
      !! Jpalm -- 07-10-2016 -- need to change carb-chem frequency call :
      !!          we don't want to call on the first time-step of all run submission, 
      !!          but only on the very first time-step, and then every month
      !!          So we call on nittrc000 if not restarted run, 
      !!          else if one month after last call.
      !!          assume one month is 30d --> 3600*24*30 : 2592000s
      !!          try to call carb-chem at 1st month's tm-stp : x * 30d + 1*rdt(i.e: mod = rdt)   
      !!          ++ need to pass carb-chem output var through restarts
      If ( ( kt == nittrc000 .AND. .NOT.ln_rsttr ) .OR. mod(kt*rdt,2592000.) == rdt ) THEN
         !!----------------------------------------------------------------------
         !! Calculate the carbonate chemistry for the whole ocean on the first
         !! simulation timestep and every month subsequently; the resulting 3D
         !! field of omega calcite is used to determine the depth of the CCD
         !!----------------------------------------------------------------------
         CALL carb_chem( kt )

      ENDIF
# endif

# if defined key_debug_medusa
      IF (lwp) write (numout,*) 'trc_bio_medusa: ready for full domain calculations'
      CALL flush(numout)
# endif 

      !!----------------------------------------------------------------------
      !! MEDUSA has unified equation through the water column
      !! (Diff. from LOBSTER which has two sets: bio- and non-bio layers) 
      !! Statement below in LOBSTER is different: DO jk = 1, jpkbm1          
      !!----------------------------------------------------------------------
      !!
      !! NOTE: the ordering of the loops below differs from that of some other
      !! models; looping over the vertical dimension is the outermost loop and
      !! this complicates some calculations (e.g. storage of vertical fluxes
      !! that can otherwise be done via a singular variable require 2D fields
      !! here); however, these issues are relatively easily resolved, but the
      !! loops CANNOT be reordered without potentially causing code efficiency
      !! problems (e.g. array indexing means that reordering the loops would
      !! require skipping between widely-spaced memory location; potentially
      !! outside those immediately cached)
      !!
      !! OPEN vertical loop
      DO jk = 1,jpk
         !! OPEN horizontal loops
         DO jj = 2,jpjm1
         DO ji = 2,jpim1
            !! OPEN wet point IF..THEN loop
            if (tmask(ji,jj,jk) == 1) then               
               !!===========================================================
               !! SETUP LOCAL GRID CELL
               !!===========================================================
               !!
               !!-----------------------------------------------------------
               !! Some notes on grid vertical structure
               !! - fsdepw(ji,jj,jk) is the depth of the upper surface of 
               !!   level jk
               !! - fsde3w(ji,jj,jk) is *approximately* the midpoint of 
               !!   level jk
               !! - fse3t(ji,jj,jk)  is the thickness of level jk
               !!-----------------------------------------------------------
               !!
               !! AXY (01/03/10): set up level depth (bottom of level)
               fdep1(ji,jj) = fsdepw(ji,jj,jk) + fse3t(ji,jj,jk)
               !! AXY (28/11/16): local seafloor depth
               !!                 previously mbathy(ji,jj) - 1, now 
               !!                 mbathy(ji,jj)
               mbathy(ji,jj) = mbathy(ji,jj)
               !!
               !! set up model tracers
               !! negative values of state variables are not allowed to
               !! contribute to the calculated fluxes
               !! non-diatom chlorophyll
               zchn(ji,jj) = max(0.,trn(ji,jj,jk,jpchn))
               !! diatom chlorophyll
               zchd(ji,jj) = max(0.,trn(ji,jj,jk,jpchd))
               !! non-diatoms
               zphn(ji,jj) = max(0.,trn(ji,jj,jk,jpphn))
               !! diatoms
               zphd(ji,jj) = max(0.,trn(ji,jj,jk,jpphd))
               !! diatom silicon
               zpds(ji,jj) = max(0.,trn(ji,jj,jk,jppds))
               !! AXY (28/01/10): probably need to take account of 
               !! chl/biomass connection
               if (zchn(ji,jj).eq.0.) zphn(ji,jj) = 0.
               if (zchd(ji,jj).eq.0.) zphd(ji,jj) = 0.
               if (zphn(ji,jj).eq.0.) zchn(ji,jj) = 0.
               if (zphd(ji,jj).eq.0.) zchd(ji,jj) = 0.
	       !! AXY (23/01/14): duh - why did I forget diatom silicon?
	       if (zpds(ji,jj).eq.0.) zphd(ji,jj) = 0.
	       if (zphd(ji,jj).eq.0.) zpds(ji,jj) = 0.
               !! microzooplankton
               zzmi(ji,jj) = max(0.,trn(ji,jj,jk,jpzmi))
               !! mesozooplankton
               zzme(ji,jj) = max(0.,trn(ji,jj,jk,jpzme))
               !! detrital nitrogen
               zdet(ji,jj) = max(0.,trn(ji,jj,jk,jpdet))
               !! dissolved inorganic nitrogen
               zdin(ji,jj) = max(0.,trn(ji,jj,jk,jpdin))
               !! dissolved silicic acid
               zsil(ji,jj) = max(0.,trn(ji,jj,jk,jpsil))
               !! dissolved "iron"
               zfer(ji,jj) = max(0.,trn(ji,jj,jk,jpfer))
# if defined key_roam
               !! detrital carbon
               zdtc(ji,jj) = max(0.,trn(ji,jj,jk,jpdtc))
               !! dissolved inorganic carbon
               zdic(ji,jj) = max(0.,trn(ji,jj,jk,jpdic))
               !! alkalinity
               zalk(ji,jj) = max(0.,trn(ji,jj,jk,jpalk))
               !! oxygen
               zoxy(ji,jj) = max(0.,trn(ji,jj,jk,jpoxy))
#  if defined key_axy_carbchem && defined key_mocsy
               !! phosphate via DIN and Redfield
               zpho(ji,jj) = max(0.,trn(ji,jj,jk,jpdin)) / 16.0
#  endif
               !!
               !! also need physical parameters for gas exchange calculations
               ztmp(ji,jj) = tsn(ji,jj,jk,jp_tem)
               zsal(ji,jj) = tsn(ji,jj,jk,jp_sal)
               !!
	       !! AXY (28/02/14): check input fields
               if (ztmp(ji,jj) .lt. -3.0 .or. ztmp(ji,jj) .gt. 40.0 ) then
                  IF(lwp) WRITE(numout,*) ' trc_bio_medusa: T WARNING 2D, ',  &
                     tsb(ji,jj,jk,jp_tem), tsn(ji,jj,jk,jp_tem), ' at (',     &
                     ji, ',', jj, ',', jk, ') at time', kt
		  IF(lwp) WRITE(numout,*) ' trc_bio_medusa: T SWITCHING 2D, ',&
                     tsn(ji,jj,jk,jp_tem), ' -> ', tsb(ji,jj,jk,jp_tem)
                  !! temperatur
                  ztmp(ji,jj) = tsb(ji,jj,jk,jp_tem)
               endif
               if (zsal(ji,jj) .lt. 0.0 .or. zsal(ji,jj) .gt. 45.0 ) then
                  IF(lwp) WRITE(numout,*) ' trc_bio_medusa: S WARNING 2D, ', &
                     tsb(ji,jj,jk,jp_sal), tsn(ji,jj,jk,jp_sal), ' at (',    &
                     ji, ',', jj, ',', jk, ') at time', kt
               endif
# else
               !! implicit detrital carbon
               zdtc(ji,jj) = zdet(ji,jj) * xthetad
# endif
# if defined key_debug_medusa
               if (idf.eq.1) then
                  !! AXY (15/01/10)
                  if (trn(ji,jj,jk,jpdin).lt.0.) then
                     IF (lwp) write (numout,*) '------------------------------'
                     IF (lwp) write (numout,*) 'NEGATIVE DIN ERROR =',        &
                        trn(ji,jj,jk,jpdin)
                     IF (lwp) write (numout,*) 'NEGATIVE DIN ERROR @',        &
                        ji, jj, jk, kt
                  endif
                  if (trn(ji,jj,jk,jpsil).lt.0.) then
                     IF (lwp) write (numout,*) '------------------------------'
                     IF (lwp) write (numout,*) 'NEGATIVE SIL ERROR =',        &
                        trn(ji,jj,jk,jpsil)
                     IF (lwp) write (numout,*) 'NEGATIVE SIL ERROR @',        &
                        ji, jj, jk, kt
                  endif
#  if defined key_roam
                  if (trn(ji,jj,jk,jpdic).lt.0.) then
                     IF (lwp) write (numout,*) '------------------------------'
                     IF (lwp) write (numout,*) 'NEGATIVE DIC ERROR =',        &
                        trn(ji,jj,jk,jpdic)
                     IF (lwp) write (numout,*) 'NEGATIVE DIC ERROR @',        &
                        ji, jj, jk, kt
                  endif
                  if (trn(ji,jj,jk,jpalk).lt.0.) then
                     IF (lwp) write (numout,*) '------------------------------'
                     IF (lwp) write (numout,*) 'NEGATIVE ALK ERROR =',        &
                        trn(ji,jj,jk,jpalk)
                     IF (lwp) write (numout,*) 'NEGATIVE ALK ERROR @',        &
                        ji, jj, jk, kt
                  endif
                  if (trn(ji,jj,jk,jpoxy).lt.0.) then
                     IF (lwp) write (numout,*) '------------------------------'
                     IF (lwp) write (numout,*) 'NEGATIVE OXY ERROR =',        &
                        trn(ji,jj,jk,jpoxy)
                     IF (lwp) write (numout,*) 'NEGATIVE OXY ERROR @',        &
                        ji, jj, jk, kt
                  endif
#  endif
               endif
# endif
# if defined key_debug_medusa
               !! report state variable values
               if (idf.eq.1.AND.idfval.eq.1) then
                  IF (lwp) write (numout,*) '------------------------------'
                  IF (lwp) write (numout,*) 'fthk(',jk,') = ', fse3t(ji,jj,jk)
                  IF (lwp) write (numout,*) 'zphn(',jk,') = ', zphn(ji,jj)
                  IF (lwp) write (numout,*) 'zphd(',jk,') = ', zphd(ji,jj)
                  IF (lwp) write (numout,*) 'zpds(',jk,') = ', zpds(ji,jj)
                  IF (lwp) write (numout,*) 'zzmi(',jk,') = ', zzmi(ji,jj)
                  IF (lwp) write (numout,*) 'zzme(',jk,') = ', zzme(ji,jj)
                  IF (lwp) write (numout,*) 'zdet(',jk,') = ', zdet(ji,jj)
                  IF (lwp) write (numout,*) 'zdin(',jk,') = ', zdin(ji,jj)
                  IF (lwp) write (numout,*) 'zsil(',jk,') = ', zsil(ji,jj)
                  IF (lwp) write (numout,*) 'zfer(',jk,') = ', zfer(ji,jj)
#  if defined key_roam
                  IF (lwp) write (numout,*) 'zdtc(',jk,') = ', zdtc(ji,jj)
                  IF (lwp) write (numout,*) 'zdic(',jk,') = ', zdic(ji,jj)
                  IF (lwp) write (numout,*) 'zalk(',jk,') = ', zalk(ji,jj)
                  IF (lwp) write (numout,*) 'zoxy(',jk,') = ', zoxy(ji,jj)                  
#  endif
               endif
# endif

# if defined key_debug_medusa
               if (idf.eq.1.AND.idfval.eq.1.AND.jk.eq.1) then
                  IF (lwp) write (numout,*) '------------------------------'
                  IF (lwp) write (numout,*) 'dust      = ', dust(ji,jj)
               endif
# endif

               !! sum tracers for inventory checks
               IF( lk_iomput ) THEN
                  IF ( med_diag%INVTN%dgsave )   THEN
                     ftot_n(ji,jj)  = ftot_n(ji,jj) +                        &
                        (fse3t(ji,jj,jk) * ( zphn(ji,jj) + zphd(ji,jj) +     &
                                             zzmi(ji,jj) + zzme(ji,jj) +     &
                                             zdet(ji,jj) + zdin(ji,jj) ) )
                  ENDIF
                  IF ( med_diag%INVTSI%dgsave )  THEN
                     ftot_si(ji,jj) = ftot_si(ji,jj) +                       & 
                        (fse3t(ji,jj,jk) * ( zpds(ji,jj) + zsil(ji,jj) ) )
                  ENDIF
                  IF ( med_diag%INVTFE%dgsave )  THEN
                     ftot_fe(ji,jj) = ftot_fe(ji,jj) +                       & 
                        (fse3t(ji,jj,jk) * ( xrfn *                          &
                                            ( zphn(ji,jj) + zphd(ji,jj) +    &
                                              zzmi(ji,jj) + zzme(ji,jj) +    &
                                              zdet(ji,jj) ) + zfer(ji,jj) ) )
                  ENDIF
# if defined key_roam
                  IF ( med_diag%INVTC%dgsave )  THEN
                     ftot_c(ji,jj)  = ftot_c(ji,jj) + & 
                        (fse3t(ji,jj,jk) * ( (xthetapn * zphn(ji,jj)) +      &
                                             (xthetapd * zphd(ji,jj)) +      &
                                             (xthetazmi * zzmi(ji,jj)) +     &
                                             (xthetazme * zzme(ji,jj)) +     &
                                             zdtc(ji,jj) + zdic(ji,jj) ) )
                  ENDIF
                  IF ( med_diag%INVTALK%dgsave ) THEN
                     ftot_a(ji,jj)  = ftot_a(ji,jj) + (fse3t(ji,jj,jk) *     &
                                                       zalk(ji,jj))
                  ENDIF
                  IF ( med_diag%INVTO2%dgsave )  THEN
                     ftot_o2(ji,jj) = ftot_o2(ji,jj) + (fse3t(ji,jj,jk) *    &
                                                        zoxy(ji,jj))
                  ENDIF
                  !!
                  !! AXY (10/11/16): CMIP6 diagnostics
                  IF ( med_diag%INTDISSIC%dgsave ) THEN
                     intdissic(ji,jj) = intdissic(ji,jj) +                   &
                                        (fse3t(ji,jj,jk) * zdic(ji,jj))
                  ENDIF
                  IF ( med_diag%INTDISSIN%dgsave ) THEN
                     intdissin(ji,jj) = intdissin(ji,jj) +                   &
                                        (fse3t(ji,jj,jk) * zdin(ji,jj))
                  ENDIF
                  IF ( med_diag%INTDISSISI%dgsave ) THEN
                     intdissisi(ji,jj) = intdissisi(ji,jj) +                 &
                                         (fse3t(ji,jj,jk) * zsil(ji,jj))
                  ENDIF
                  IF ( med_diag%INTTALK%dgsave ) THEN
                     inttalk(ji,jj) = inttalk(ji,jj) +                       &
                                      (fse3t(ji,jj,jk) * zalk(ji,jj))
                  ENDIF
                  IF ( med_diag%O2min%dgsave ) THEN
                     if ( zoxy(ji,jj) < o2min(ji,jj) ) then
                        o2min(ji,jj)  = zoxy(ji,jj)
                        IF ( med_diag%ZO2min%dgsave ) THEN
                           !! layer midpoint
                           zo2min(ji,jj) = (fsdepw(ji,jj,jk) +               &
                                            fdep1(ji,jj)) / 2.0
                        ENDIF
                     endif
                  ENDIF
# endif
               ENDIF

               CALL flush(numout)

            ENDIF
         ENDDO
         ENDDO

         !!----------------------------------------------------------------
         !! Calculate air-sea gas exchange and river inputs
         !!----------------------------------------------------------------
         IF ( jk == 1 ) THEN
            call air_sea( kt )
         END IF

# if defined key_roam

! Moved from above - marc 21/4/17
! I think this should be moved into diagnostics at bottom - it
! doesn't like it is used anyway else - marc 21/4/17
         DO jj = 2,jpjm1
         DO ji = 2,jpim1
            !! OPEN wet point IF..THEN loop
            if (tmask(ji,jj,jk) == 1) then

               !! AXY (11/11/16): CMIP6 oxygen saturation 3D diagnostic
               IF ( med_diag%O2SAT3%dgsave ) THEN
                  call oxy_sato( ztmp(ji,jj), zsal(ji,jj), f_o2sat3 )
                  o2sat3(ji, jj, jk) = f_o2sat3
               ENDIF
            ENDIF
         ENDDO
         ENDDO
# endif

         !!------------------------------------------------------------------
         !! Phytoplankton growth, zooplankton grazing and miscellaneous
         !! plankton losses. 
         !!------------------------------------------------------------------
         CALL plankton( jk )

! Detritus process - marc

         DO jj = 2,jpjm1
         DO ji = 2,jpim1
            !! OPEN wet point IF..THEN loop
            if (tmask(ji,jj,jk) == 1) then

               !!----------------------------------------------------------------------
               !! Detritus remineralisation
               !! Constant or temperature-dependent
               !!----------------------------------------------------------------------
               !!
               if (jmd.eq.1) then
                  !! temperature-dependent
                  fdd(ji,jj)  = xmd  * fun_T(ji,jj) * zdet(ji,jj)
# if defined key_roam
                  fddc(ji,jj) = xmdc * fun_T(ji,jj) * zdtc(ji,jj)
# endif
               elseif (jmd.eq.2) then
                  !! AXY (16/05/13): add in Q10-based parameterisation (def in nmlst)
                  !! temperature-dependent
                  fdd(ji,jj)  = xmd  * fun_Q10(ji,jj) * zdet(ji,jj)
#if defined key_roam
                  fddc(ji,jj) = xmdc * fun_Q10(ji,jj) * zdtc(ji,jj)
#endif
               else
                  !! temperature-independent
                  fdd(ji,jj)  = xmd  * zdet(ji,jj)
# if defined key_roam
                  fddc(ji,jj) = xmdc * zdtc(ji,jj)
# endif
               endif
               !!
               !! AXY (22/07/09): accelerate detrital remineralisation in the bottom box
               if ((jk.eq.mbathy(ji,jj)) .and. jsfd.eq.1) then
                  fdd(ji,jj)  = 1.0  * zdet(ji,jj)
# if defined key_roam
                  fddc(ji,jj) = 1.0  * zdtc(ji,jj)
# endif
               endif
               
# if defined key_debug_medusa
               !! report plankton mortality and remineralisation
               if (idf.eq.1.AND.idfval.eq.1) then
                  IF (lwp) write (numout,*) '------------------------------'
                  IF (lwp) write (numout,*) 'fdpn2(',jk,') = ', fdpn2(ji,jj)
                  IF (lwp) write (numout,*) 'fdpd2(',jk,') = ', fdpd2(ji,jj)
                  IF (lwp) write (numout,*) 'fdpds2(',jk,')= ', fdpds2(ji,jj)
                  IF (lwp) write (numout,*) 'fdzmi2(',jk,')= ', fdzmi2(ji,jj)
                  IF (lwp) write (numout,*) 'fdzme2(',jk,')= ', fdzme2(ji,jj)
                  IF (lwp) write (numout,*) 'fdpn(',jk,')  = ', fdpn(ji,jj)
                  IF (lwp) write (numout,*) 'fdpd(',jk,')  = ', fdpd(ji,jj)
                  IF (lwp) write (numout,*) 'fdpds(',jk,') = ', fdpds(ji,jj)
                  IF (lwp) write (numout,*) 'fdzmi(',jk,') = ', fdzmi(ji,jj)
                  IF (lwp) write (numout,*) 'fdzme(',jk,') = ', fdzme(ji,jj)
                  IF (lwp) write (numout,*) 'fdd(',jk,')   = ', fdd(ji,jj)
#  if defined key_roam
                  IF (lwp) write (numout,*) 'fddc(',jk,')  = ', fddc(ji,jj)
#  endif
               endif
# endif

               !!----------------------------------------------------------------------
               !! Detritus addition to benthos
               !! If activated, slow detritus in the bottom box will enter the
               !! benthic pool
               !!----------------------------------------------------------------------
               !!
               if ((jk.eq.mbathy(ji,jj)) .and. jorgben.eq.1) then
                  !! this is the BOTTOM OCEAN BOX -> into the benthic pool!
                  !!
                  f_sbenin_n(ji,jj)  = (zdet(ji,jj) * vsed * 86400.)
                  f_sbenin_fe(ji,jj) = (zdet(ji,jj) * vsed * 86400. * xrfn)
# if defined key_roam
                  f_sbenin_c(ji,jj)  = (zdtc(ji,jj) * vsed * 86400.)
# else
                  f_sbenin_c(ji,jj)  = (zdet(ji,jj) * vsed * 86400. * xthetad)
# endif
               endif

! MAYBE BUT A BREAK IN HERE, IRON CHEMISTRY AND SCAVENGING - marc 20/4/17 
! (detritus processes is 74 lines)
            ENDIF
         ENDDO
         ENDDO

         DO jj = 2,jpjm1
         DO ji = 2,jpim1
            !! OPEN wet point IF..THEN loop
            if (tmask(ji,jj,jk) == 1) then

               !!----------------------------------------------------------------------
               !! Iron chemistry and fractionation
               !! following the Parekh et al. (2004) scheme adopted by the Met.
               !! Office, Medusa models total iron but considers "free" and
               !! ligand-bound forms for the purposes of scavenging (only "free"
               !! iron can be scavenged
               !!----------------------------------------------------------------------
               !!
               !! total iron concentration (mmol Fe / m3 -> umol Fe / m3)
               xFeT        = zfer(ji,jj) * 1.e3
               !!
               !! calculate fractionation (based on Diat-HadOCC; in turn based on Parekh et al., 2004)
               xb_coef_tmp = xk_FeL * (xLgT - xFeT) - 1.0
               xb2M4ac     = max(((xb_coef_tmp * xb_coef_tmp) + (4.0 * xk_FeL * xLgT)), 0.0)
               !!
               !! "free" ligand concentration
               xLgF        = 0.5 * (xb_coef_tmp + (xb2M4ac**0.5)) / xk_FeL
               !!
               !! ligand-bound iron concentration
               xFeL        = xLgT - xLgF
               !!
               !! "free" iron concentration (and convert to mmol Fe / m3)
               xFeF        = (xFeT - xFeL) * 1.e-3
               xFree(ji,jj)= xFeF / (zfer(ji,jj) + tiny(zfer(ji,jj)))
               !!
               !! scavenging of iron (multiple schemes); I'm only really happy with the 
               !! first one at the moment - the others involve assumptions (sometimes
               !! guessed at by me) that are potentially questionable
               !!
               if (jiron.eq.1) then
                  !!----------------------------------------------------------------------
                  !! Scheme 1: Dutkiewicz et al. (2005)
                  !! This scheme includes a single scavenging term based solely on a
                  !! fixed rate and the availablility of "free" iron
                  !!----------------------------------------------------------------------
                  !!
                  ffescav(ji,jj)     = xk_sc_Fe * xFeF                     ! = mmol/m3/d
                  !!
                  !!----------------------------------------------------------------------
                  !!
                  !! Mick's code contains a further (optional) implicit "scavenging" of 
                  !! iron that sets an upper bound on "free" iron concentration, and 
                  !! essentially caps the concentration of total iron as xFeL + "free" 
                  !! iron; since the former is constrained by a fixed total ligand 
                  !! concentration (= 1.0 umol/m3), and the latter isn't allowed above 
                  !! this upper bound, total iron is constrained to a maximum of ...
                  !!
                  !!    xFeL + min(xFeF, 0.3 umol/m3) = 1.0 + 0.3 = 1.3 umol / m3
                  !! 
                  !! In Mick's code, the actual value of total iron is reset to this
                  !! sum (i.e. TFe = FeL + Fe'; but Fe' <= 0.3 umol/m3); this isn't
                  !! our favoured approach to tracer updating here (not least because
                  !! of the leapfrog), so here the amount scavenged is augmented by an
                  !! additional amount that serves to drag total iron back towards that
                  !! expected from this limitation on iron concentration ...
                  !!
                  xmaxFeF     = min((xFeF * 1.e3), 0.3)             ! = umol/m3
                  !!
                  !! Here, the difference between current total Fe and (FeL + Fe') is
                  !! calculated and added to the scavenging flux already calculated
                  !! above ...
                  !!
                  fdeltaFe    = (xFeT - (xFeL + xmaxFeF)) * 1.e-3   ! = mmol/m3
                  !!
                  !! This assumes that the "excess" iron is dissipated with a time-
                  !! scale of 1 day; seems reasonable to me ... (famous last words)
                  !!
                  ffescav(ji,jj)     = ffescav(ji,jj) + fdeltaFe                  ! = mmol/m3/d
                  !!
# if defined key_deep_fe_fix
                  !! AXY (17/01/13)
                  !! stop scavenging for iron concentrations below 0.5 umol / m3
                  !! at depths greater than 1000 m; this aims to end MEDUSA's
                  !! continual loss of iron at depth without impacting things
                  !! at the surface too much; the justification for this is that
                  !! it appears to be what Mick Follows et al. do in their work
                  !! (as evidenced by the iron initial condition they supplied
                  !! me with); to be honest, it looks like Follow et al. do this
                  !! at shallower depths than 1000 m, but I'll stick with this
                  !! for now; I suspect that this seemingly arbitrary approach
                  !! effectively "parameterises" the particle-based scavenging
                  !! rates that other models use (i.e. at depth there are no
                  !! sinking particles, so scavenging stops); it might be fun
                  !! justifying this in a paper though!
                  !!
                  if ((fsdepw(ji,jj,jk).gt.1000.) .and. (xFeT.lt.0.5)) then
                     ffescav(ji,jj) = 0.
                  endif
# endif
                  !!
               elseif (jiron.eq.2) then
                  !!----------------------------------------------------------------------
                  !! Scheme 2: Moore et al. (2004)
                  !! This scheme includes a single scavenging term that accounts for
                  !! both suspended and sinking particles in the water column; this
                  !! term scavenges total iron rather than "free" iron
                  !!----------------------------------------------------------------------
                  !!
                  !! total iron concentration (mmol Fe / m3 -> umol Fe / m3)
                  xFeT        = zfer(ji,jj) * 1.e3
                  !!
                  !! this has a base scavenging rate (12% / y) which is modified by local
                  !! particle concentration and sinking flux (and dust - but I'm ignoring
                  !! that here for now) and which is accelerated when Fe concentration gets
                  !! 0.6 nM (= 0.6 umol/m3 = 0.0006 mmol/m3), and decreased as concentrations
                  !! below 0.4 nM (= 0.4 umol/m3 = 0.0004 mmol/m3)
                  !!
                  !! base scavenging rate (0.12 / y)
                  fbase_scav = 0.12 / 365.25
                  !!
                  !! calculate sinking particle part of scaling factor
                  !! this takes local fast sinking carbon (mmol C / m2 / d) and
                  !! gets it into nmol C / cm3 / s ("rdt" below is the number of seconds in
                  !! a model timestep)
                  !!
                  !! fscal_sink = ffastc(ji,jj) * 1.e2 / (86400.)
                  !!
                  !! ... actually, re-reading Moore et al.'s equations, it looks like he uses
                  !! his sinking flux directly, without scaling it by time-step or anything,
                  !! so I'll copy this here ...
                  !!
                  fscal_sink = ffastc(ji,jj) * 1.e2
                  !!
                  !! calculate particle part of scaling factor
                  !! this totals up the carbon in suspended particles (Pn, Pd, Zmi, Zme, D),
                  !! which comes out in mmol C / m3 (= nmol C / cm3), and then multiplies it
                  !! by a magic factor, 0.002, to get it into nmol C / cm2 / s
                  !!
                  fscal_part = ((xthetapn * zphn(ji,jj)) + (xthetapd * zphd(ji,jj)) + (xthetazmi * zzmi(ji,jj)) + &
                  (xthetazme * zzme(ji,jj)) + (xthetad * zdet(ji,jj))) * 0.002
                  !!
                  !! calculate scaling factor for base scavenging rate
                  !! this uses the (now correctly scaled) sinking flux and standing
                  !! particle concentration, divides through by some sort of reference
                  !! value (= 0.0066 nmol C / cm2 / s) and then uses this, or not if its
                  !! too high, to rescale the base scavenging rate
                  !!
                  fscal_scav = fbase_scav * min(((fscal_sink + fscal_part) / 0.0066), 4.0)
                  !!
                  !! the resulting scavenging rate is then scaled further according to the
                  !! local iron concentration (i.e. diminished in low iron regions; enhanced
                  !! in high iron regions; less alone in intermediate iron regions)
                  !!
                  if (xFeT.lt.0.4) then
                     !!
                     !! low iron region
                     !!
                     fscal_scav = fscal_scav * (xFeT / 0.4)
                     !!
                  elseif (xFeT.gt.0.6) then
                     !!
                     !! high iron region
                     !!
                     fscal_scav = fscal_scav + ((xFeT / 0.6) * (6.0 / 1.4))
                     !!
                  else
                     !!
                     !! intermediate iron region: do nothing
                     !!
                  endif
                  !! 
                  !! apply the calculated scavenging rate ...
                  !!
                  ffescav(ji,jj) = fscal_scav * zfer(ji,jj)
                  !!
               elseif (jiron.eq.3) then
                  !!----------------------------------------------------------------------
                  !! Scheme 3: Moore et al. (2008)
                  !! This scheme includes a single scavenging term that accounts for
                  !! sinking particles in the water column, and includes organic C,
                  !! biogenic opal, calcium carbonate and dust in this (though the
                  !! latter is ignored here until I work out what units the incoming
                  !! "dust" flux is in); this term scavenges total iron rather than 
                  !! "free" iron
                  !!----------------------------------------------------------------------
                  !!
                  !! total iron concentration (mmol Fe / m3 -> umol Fe / m3)
                  xFeT        = zfer(ji,jj) * 1.e3
                  !!
                  !! this has a base scavenging rate which is modified by local
                  !! particle sinking flux (including dust - but I'm ignoring that 
                  !! here for now) and which is accelerated when Fe concentration
                  !! is > 0.6 nM (= 0.6 umol/m3 = 0.0006 mmol/m3), and decreased as 
                  !! concentrations < 0.5 nM (= 0.5 umol/m3 = 0.0005 mmol/m3)
                  !!
                  !! base scavenging rate (Fe_b in paper; units may be wrong there)
                  fbase_scav = 0.00384 ! (ng)^-1 cm
                  !!
                  !! calculate sinking particle part of scaling factor; this converts
                  !! mmol / m2 / d fluxes of organic carbon, silicon and calcium
                  !! carbonate into ng / cm2 / s fluxes; it is assumed here that the
                  !! mass conversions simply consider the mass of the main element
                  !! (C, Si and Ca) and *not* the mass of the molecules that they are
                  !! part of; Moore et al. (2008) is unclear on the conversion that
                  !! should be used
                  !!
                  !! milli -> nano; mol -> gram; /m2 -> /cm2; /d -> /s
                  fscal_csink  = (ffastc(ji,jj)  * 1.e6 * xmassc  * 1.e-4 / 86400.)      ! ng C  / cm2 / s
                  fscal_sisink = (ffastsi(ji,jj) * 1.e6 * xmasssi * 1.e-4 / 86400.)      ! ng Si / cm2 / s
                  fscal_casink = (ffastca(ji,jj) * 1.e6 * xmassca * 1.e-4 / 86400.)      ! ng Ca / cm2 / s
                  !! 
                  !! sum up these sinking fluxes and convert to ng / cm by dividing
                  !! through by a sinking rate of 100 m / d = 1.157 cm / s
                  fscal_sink   = ((fscal_csink * 6.) + fscal_sisink + fscal_casink) / &
                  (100. * 1.e3 / 86400)                                                  ! ng / cm
                  !!
                  !! now calculate the scavenging rate based upon the base rate and
                  !! this particle flux scaling; according to the published units,
                  !! the result actually has *no* units, but as it must be expressed
                  !! per unit time for it to make any sense, I'm assuming a missing
                  !! "per second"
                  fscal_scav = fbase_scav * fscal_sink                                   ! / s
                  !!
                  !! the resulting scavenging rate is then scaled further according to the
                  !! local iron concentration (i.e. diminished in low iron regions; enhanced
                  !! in high iron regions; less alone in intermediate iron regions)
                  !!
                  if (xFeT.lt.0.5) then
                     !!
                     !! low iron region (0.5 instead of the 0.4 in Moore et al., 2004)
                     !!
                     fscal_scav = fscal_scav * (xFeT / 0.5)
                     !!
                  elseif (xFeT.gt.0.6) then
                     !!
                     !! high iron region (functional form different in Moore et al., 2004)
                     !!
                     fscal_scav = fscal_scav + ((xFeT - 0.6) * 0.00904)
                     !!
                  else
                     !!
                     !! intermediate iron region: do nothing
                     !!
                  endif
                  !! 
                  !! apply the calculated scavenging rate ...
                  !!
                  ffescav(ji,jj) = fscal_scav * zfer(ji,jj)
                  !!
               elseif (jiron.eq.4) then
                  !!----------------------------------------------------------------------
                  !! Scheme 4: Galbraith et al. (2010)
                  !! This scheme includes two scavenging terms, one for organic,
                  !! particle-based scavenging, and another for inorganic scavenging;
                  !! both terms scavenge "free" iron only
                  !!----------------------------------------------------------------------
                  !!
                  !! Galbraith et al. (2010) present a more straightforward outline of 
                  !! the scheme in Parekh et al. (2005) ...
                  !! 
                  !! sinking particulate carbon available for scavenging
                  !! this assumes a sinking rate of 100 m / d (Moore & Braucher, 2008),
                  xCscav1     = (ffastc(ji,jj) * xmassc) / 100. ! = mg C / m3
                  !! 
                  !! scale by Honeyman et al. (1981) exponent coefficient
                  !! multiply by 1.e-3 to express C flux in g C rather than mg C
                  xCscav2     = (xCscav1 * 1.e-3)**0.58
                  !!
                  !! multiply by Galbraith et al. (2010) scavenging rate
                  xk_org      = 0.5 ! ((g C m/3)^-1) / d
                  xORGscav    = xk_org * xCscav2 * xFeF
                  !!
                  !! Galbraith et al. (2010) also include an inorganic bit ...
                  !! 
                  !! this occurs at a fixed rate, again based on the availability of
                  !! "free" iron
                  !!
                  !! k_inorg  = 1000 d**-1 nmol Fe**-0.5 kg**-0.5
                  !!
                  !! to implement this here, scale xFeF by 1026 to put in units of
                  !! umol/m3 which approximately equal nmol/kg
                  !!
                  xk_inorg    = 1000. ! ((nmol Fe / kg)^1.5)
                  xINORGscav  = (xk_inorg * (xFeF * 1026.)**1.5) * 1.e-3
                  !!
                  !! sum these two terms together
                  ffescav(ji,jj)     = xORGscav + xINORGscav
               else
                  !!----------------------------------------------------------------------
                  !! No Scheme: you coward!
                  !! This scheme puts its head in the sand and eskews any decision about
                  !! how iron is removed from the ocean; prepare to get deluged in iron
                  !! you fool!
                  !!----------------------------------------------------------------------
                  ffescav(ji,jj)     = 0.
               endif

               !!----------------------------------------------------------------------
               !! Other iron cycle processes
               !!----------------------------------------------------------------------
               !!
               !! aeolian iron deposition
               if (jk.eq.1) then
                  !! zirondep   is in mmol-Fe / m2 / day
                  !! ffetop(ji,jj)     is in mmol-dissolved-Fe / m3 / day
                  ffetop(ji,jj)  = zirondep(ji,jj) * xfe_sol / fse3t(ji,jj,jk) 
               else
                  ffetop(ji,jj)  = 0.0
               endif
               !!
               !! seafloor iron addition
               !! AXY (10/07/12): amended to only apply sedimentary flux up to ~500 m down
               !! if (jk.eq.(mbathy(ji,jj)-1).AND.jk.lt.i1100) then
               if ((jk.eq.mbathy(ji,jj)).AND.jk.le.i0500) then
                  !! Moore et al. (2004) cite a coastal California value of 5 umol/m2/d, but adopt a
                  !! global value of 2 umol/m2/d for all areas < 1100 m; here we use this latter value
                  !! but apply it everywhere
                  !! AXY (21/07/09): actually, let's just apply it below 1100 m (levels 1-37)
                  ffebot(ji,jj)  = (xfe_sed / fse3t(ji,jj,jk))
               else
                  ffebot(ji,jj)  = 0.0
               endif

               !! AXY (16/12/09): remove iron addition/removal processes
               !! For the purposes of the quarter degree run, the iron cycle is being pegged to the
               !! initial condition supplied by Mick Follows via restoration with a 30 day period;
               !! iron addition at the seafloor is still permitted with the idea that this extra
               !! iron will be removed by the restoration away from the source
               !! ffescav(ji,jj) = 0.0
               !! ffetop(ji,jj)  = 0.0
               !! ffebot(ji,jj)  = 0.0

# if defined key_debug_medusa
               !! report miscellaneous calculations
               if (idf.eq.1.AND.idfval.eq.1) then
                  IF (lwp) write (numout,*) '------------------------------'
                  IF (lwp) write (numout,*) 'xfe_sol  = ', xfe_sol
                  IF (lwp) write (numout,*) 'xfe_mass = ', xfe_mass
                  IF (lwp) write (numout,*) 'ffetop(',jk,')  = ', ffetop(ji,jj)
                  IF (lwp) write (numout,*) 'ffebot(',jk,')  = ', ffebot(ji,jj)
                  IF (lwp) write (numout,*) 'xFree(',jk,')   = ', xFree(ji,jj)
                  IF (lwp) write (numout,*) 'ffescav(',jk,') = ', ffescav(ji,jj)
               endif
# endif

! MAYBE BUT A BREAK IN HERE, MISCELLANEOUS PROCESSES - marc 20/4/17 
! (iron chemistry and scavenging is 340 lines)
            ENDIF
         ENDDO
         ENDDO

         DO jj = 2,jpjm1
         DO ji = 2,jpim1
            !! OPEN wet point IF..THEN loop
            if (tmask(ji,jj,jk) == 1) then

               !!----------------------------------------------------------------------
               !! Miscellaneous
               !!----------------------------------------------------------------------
               !!
               !! diatom frustule dissolution
               fsdiss(ji,jj)  = xsdiss * zpds(ji,jj)

# if defined key_debug_medusa
               !! report miscellaneous calculations
               if (idf.eq.1.AND.idfval.eq.1) then
                  IF (lwp) write (numout,*) '------------------------------'
                  IF (lwp) write (numout,*) 'fsdiss(',jk,')  = ', fsdiss(ji,jj)
               endif
# endif

               !!----------------------------------------------------------------------
               !! Slow detritus creation
               !!----------------------------------------------------------------------
               !! this variable integrates the creation of slow sinking detritus
               !! to allow the split between fast and slow detritus to be 
               !! diagnosed
               fslown(ji,jj)  = fdpn(ji,jj) + fdzmi(ji,jj) + ((1.0 - xfdfrac1) * fdpd(ji,jj)) + &
               ((1.0 - xfdfrac2) * fdzme(ji,jj)) + ((1.0 - xbetan) * (finmi(ji,jj) + finme(ji,jj)))
               !!
               !! this variable records the slow detrital sinking flux at this
               !! particular depth; it is used in the output of this flux at
               !! standard depths in the diagnostic outputs; needs to be
               !! adjusted from per second to per day because of parameter vsed
               fslownflux(ji,jj) = zdet(ji,jj) * vsed * 86400.
# if defined key_roam
               !!
               !! and the same for detrital carbon
               fslowc(ji,jj)  = (xthetapn * fdpn(ji,jj)) + (xthetazmi * fdzmi(ji,jj)) + &
               (xthetapd * (1.0 - xfdfrac1) * fdpd(ji,jj)) + &
               (xthetazme * (1.0 - xfdfrac2) * fdzme(ji,jj)) + &
               ((1.0 - xbetac) * (ficmi(ji,jj) + ficme(ji,jj)))
               !!
               !! this variable records the slow detrital sinking flux at this
               !! particular depth; it is used in the output of this flux at
               !! standard depths in the diagnostic outputs; needs to be
               !! adjusted from per second to per day because of parameter vsed
               fslowcflux(ji,jj) = zdtc(ji,jj) * vsed * 86400.
# endif

               !!----------------------------------------------------------------------
               !! Nutrient regeneration
               !! this variable integrates total nitrogen regeneration down the
               !! watercolumn; its value is stored and output as a 2D diagnostic;
               !! the corresponding dissolution flux of silicon (from sources
               !! other than fast detritus) is also integrated; note that,
               !! confusingly, the linear loss terms from plankton compartments
               !! are labelled as fdX2 when one might have expected fdX or fdX1
               !!----------------------------------------------------------------------
               !!
               !! nitrogen
               fregen(ji,jj)   = (( (xphi * (fgmipn(ji,jj) + fgmid(ji,jj))) +                &  ! messy feeding
               (xphi * (fgmepn(ji,jj) + fgmepd(ji,jj) + fgmezmi(ji,jj) + fgmed(ji,jj))) +           &  ! messy feeding
               fmiexcr(ji,jj) + fmeexcr(ji,jj) + fdd(ji,jj) +                                &  ! excretion + D remin.
               fdpn2(ji,jj) + fdpd2(ji,jj) + fdzmi2(ji,jj) + fdzme2(ji,jj)) * fse3t(ji,jj,jk))                    ! linear mortality
               !!
               !! silicon
               fregensi(ji,jj) = (( fsdiss(ji,jj) + ((1.0 - xfdfrac1) * fdpds(ji,jj)) +      &  ! dissolution + non-lin. mortality
               ((1.0 - xfdfrac3) * fgmepds(ji,jj)) +                           &  ! egestion by zooplankton
               fdpds2(ji,jj)) * fse3t(ji,jj,jk))                                             ! linear mortality
# if defined key_roam
               !!
               !! carbon
! Doesn't look this is used - marc 10/4/17
!               fregenc(ji,jj)  = (( (xphi * ((xthetapn * fgmipn(ji,jj)) + fgmidc(ji,jj))) +  &  ! messy feeding
!               (xphi * ((xthetapn * fgmepn(ji,jj)) + (xthetapd * fgmepd(ji,jj)) +     &  ! messy feeding
!               (xthetazmi * fgmezmi(ji,jj)) + fgmedc(ji,jj))) +                       &  ! messy feeding
!               fmiresp(ji,jj) + fmeresp(ji,jj) + fddc(ji,jj) +                               &  ! respiration + D remin.
!               (xthetapn * fdpn2(ji,jj)) + (xthetapd * fdpd2(ji,jj)) +                &  ! linear mortality
!               (xthetazmi * fdzmi2(ji,jj)) + (xthetazme * fdzme2(ji,jj))) * fse3t(ji,jj,jk))        ! linear mortality
# endif

! MAYBE BUT A BREAK IN HERE, FAST-SINKINIG DETRITUS - marc 20/4/17 
! (miscellaneous processes is 79 lines)
            ENDIF
         ENDDO
         ENDDO

         DO jj = 2,jpjm1
         DO ji = 2,jpim1
            !! OPEN wet point IF..THEN loop
            if (tmask(ji,jj,jk) == 1) then

               !!----------------------------------------------------------------------
               !! Fast-sinking detritus terms
               !! "local" variables declared so that conservation can be checked;
               !! the calculated terms are added to the fast-sinking flux later on
               !! only after the flux entering this level has experienced some
               !! remineralisation
               !! note: these fluxes need to be scaled by the level thickness
               !!----------------------------------------------------------------------
               !!
               !! nitrogen:   diatom and mesozooplankton mortality
               ftempn(ji,jj)         = b0 * ((xfdfrac1 * fdpd(ji,jj))  + (xfdfrac2 * fdzme(ji,jj)))
               !!
               !! silicon:    diatom mortality and grazed diatoms
               ftempsi(ji,jj)        = b0 * ((xfdfrac1 * fdpds(ji,jj)) + (xfdfrac3 * fgmepds(ji,jj)))
               !!
               !! iron:       diatom and mesozooplankton mortality
               ftempfe(ji,jj)        = b0 * (((xfdfrac1 * fdpd(ji,jj)) + (xfdfrac2 * fdzme(ji,jj))) * xrfn)
               !!
               !! carbon:     diatom and mesozooplankton mortality
               ftempc(ji,jj)         = b0 * ((xfdfrac1 * xthetapd * fdpd(ji,jj)) + & 
                                (xfdfrac2 * xthetazme * fdzme(ji,jj)))
               !!
# if defined key_roam
               if (jrratio.eq.0) then
                  !! CaCO3:      latitudinally-based fraction of total primary production
                  !!               0.10 at equator; 0.02 at pole
                  fcaco3(ji,jj)         = xcaco3a + ((xcaco3b - xcaco3a) * ((90.0 - abs(gphit(ji,jj))) / 90.0))
               elseif (jrratio.eq.1) then
                  !! CaCO3:      Ridgwell et al. (2007) submodel, version 1
                  !!             this uses SURFACE omega calcite to regulate rain ratio
                  if (f_omcal(ji,jj).ge.1.0) then
                     fq1 = (f_omcal(ji,jj) - 1.0)**0.81
                  else
                     fq1 = 0.
                  endif
                  fcaco3(ji,jj) = xridg_r0 * fq1
               elseif (jrratio.eq.2) then
                  !! CaCO3:      Ridgwell et al. (2007) submodel, version 2
                  !!             this uses FULL 3D omega calcite to regulate rain ratio
                  if (f3_omcal(ji,jj,jk).ge.1.0) then
                     fq1 = (f3_omcal(ji,jj,jk) - 1.0)**0.81
                  else
                     fq1 = 0.
                  endif
                  fcaco3(ji,jj) = xridg_r0 * fq1
               endif
# else
               !! CaCO3:      latitudinally-based fraction of total primary production
               !!               0.10 at equator; 0.02 at pole
               fcaco3(ji,jj)         = xcaco3a + ((xcaco3b - xcaco3a) * ((90.0 - abs(gphit(ji,jj))) / 90.0))
# endif
               !! AXY (09/03/09): convert CaCO3 production from function of 
               !! primary production into a function of fast-sinking material; 
               !! technically, this is what Dunne et al. (2007) do anyway; they 
               !! convert total primary production estimated from surface 
               !! chlorophyll to an export flux for which they apply conversion 
               !! factors to estimate the various elemental fractions (Si, Ca)
               ftempca(ji,jj)        = ftempc(ji,jj) * fcaco3(ji,jj)

# if defined key_debug_medusa
               !! integrate total fast detritus production
               if (idf.eq.1) then
                  fifd_n(ji,jj)  = fifd_n(ji,jj)  + (ftempn(ji,jj)  * fse3t(ji,jj,jk))
                  fifd_si(ji,jj) = fifd_si(ji,jj) + (ftempsi(ji,jj) * fse3t(ji,jj,jk))
                  fifd_fe(ji,jj) = fifd_fe(ji,jj) + (ftempfe(ji,jj) * fse3t(ji,jj,jk))
#  if defined key_roam
                  fifd_c(ji,jj)  = fifd_c(ji,jj)  + (ftempc(ji,jj)  * fse3t(ji,jj,jk))
#  endif
               endif

               !! report quantities of fast-sinking detritus for each component
               if (idf.eq.1.AND.idfval.eq.1) then
                  IF (lwp) write (numout,*) '------------------------------'
                  IF (lwp) write (numout,*) 'fdpd(',jk,')    = ', fdpd(ji,jj)
                  IF (lwp) write (numout,*) 'fdzme(',jk,')   = ', fdzme(ji,jj)
                  IF (lwp) write (numout,*) 'ftempn(',jk,')  = ', ftempn(ji,jj)
                  IF (lwp) write (numout,*) 'ftempsi(',jk,') = ', ftempsi(ji,jj)
                  IF (lwp) write (numout,*) 'ftempfe(',jk,') = ', ftempfe(ji,jj)
                  IF (lwp) write (numout,*) 'ftempc(',jk,')  = ', ftempc(ji,jj)
                  IF (lwp) write (numout,*) 'ftempca(',jk,') = ', ftempca(ji,jj)
                  IF (lwp) write (numout,*) 'flat(',jk,')    = ', abs(gphit(ji,jj))
                  IF (lwp) write (numout,*) 'fcaco3(',jk,')  = ', fcaco3(ji,jj)
               endif
# endif

               !!----------------------------------------------------------------------
               !! This version of MEDUSA offers a choice of three methods for
               !! handling the remineralisation of fast detritus.  All three
               !! do so in broadly the same way:
               !!
               !!   1.  Fast detritus is stored as a 2D array                   [ ffastX  ]
               !!   2.  Fast detritus is added level-by-level                   [ ftempX  ]
               !!   3.  Fast detritus is not remineralised in the top box       [ freminX ]
               !!   4.  Remaining fast detritus is remineralised in the bottom  [ fsedX   ]
               !!       box
               !!
               !! The three remineralisation methods are:
               !!   
               !!   1.  Ballast model (i.e. that published in Yool et al., 2011)
               !!  (1b. Ballast-sans-ballast model)
               !!   2.  Martin et al. (1987)
               !!   3.  Henson et al. (2011)
               !! 
               !! The first of these couples C, N and Fe remineralisation to
               !! the remineralisation of particulate Si and CaCO3, but the 
               !! latter two treat remineralisation of C, N, Fe, Si and CaCO3
               !! completely separately.  At present a switch within the code
               !! regulates which submodel is used, but this should be moved
               !! to the namelist file.
               !! 
               !! The ballast-sans-ballast submodel is an original development
               !! feature of MEDUSA in which the ballast submodel's general
               !! framework and parameterisation is used, but in which there
               !! is no protection of organic material afforded by ballasting
               !! minerals.  While similar, it is not the same as the Martin 
               !! et al. (1987) submodel.
               !!
               !! Since the three submodels behave the same in terms of
               !! accumulating sinking material and remineralising it all at
               !! the seafloor, these portions of the code below are common to
               !! all three.
               !!----------------------------------------------------------------------

               if (jexport.eq.1) then
                  !!======================================================================
                  !! BALLAST SUBMODEL
                  !!======================================================================
                  !! 
                  !!----------------------------------------------------------------------
                  !! Fast-sinking detritus fluxes, pt. 1: REMINERALISATION
                  !! aside from explicitly modelled, slow-sinking detritus, the
                  !! model includes an implicit representation of detrital
                  !! particles that sink too quickly to be modelled with
                  !! explicit state variables; this sinking flux is instead
                  !! instantaneously remineralised down the water column using
                  !! the version of Armstrong et al. (2002)'s ballast model
                  !! used by Dunne et al. (2007); the version of this model
                  !! here considers silicon and calcium carbonate ballast
                  !! minerals; this section of the code redistributes the fast
                  !! sinking material generated locally down the water column;
                  !! this differs from Dunne et al. (2007) in that fast sinking
                  !! material is distributed at *every* level below that it is
                  !! generated, rather than at every level below some fixed
                  !! depth; this scheme is also different in that sinking material 
                  !! generated in one level is aggregated with that generated by
                  !! shallower levels; this should make the ballast model more
                  !! self-consistent (famous last words)
                  !!----------------------------------------------------------------------
                  !!
                  if (jk.eq.1) then
                     !! this is the SURFACE OCEAN BOX (no remineralisation)
                     !!
                     freminc(ji,jj)  = 0.0
                     freminn(ji,jj)  = 0.0
                     freminfe(ji,jj) = 0.0
                     freminsi(ji,jj) = 0.0
                     freminca(ji,jj) = 0.0
                  elseif (jk.le.mbathy(ji,jj)) then
                     !! this is an OCEAN BOX (remineralise some material)
                     !!
                     !! set up CCD depth to be used depending on user choice
                     if (jocalccd.eq.0) then
                        !! use default CCD field
                        fccd_dep(ji,jj) = ocal_ccd(ji,jj)
                     elseif (jocalccd.eq.1) then
                        !! use calculated CCD field
                        fccd_dep(ji,jj) = f2_ccd_cal(ji,jj)
                     endif
                     !!
                     !! === organic carbon ===
                     fq0      = ffastc(ji,jj)                        !! how much organic C enters this box        (mol)
                     if (iball.eq.1) then
                        fq1      = (fq0 * xmassc)                    !! how much it weighs                        (mass)
                        fq2      = (ffastca(ji,jj) * xmassca)        !! how much CaCO3 enters this box            (mass)
                        fq3      = (ffastsi(ji,jj) * xmasssi)        !! how much  opal enters this box            (mass)
                        fq4      = (fq2 * xprotca) + (fq3 * xprotsi) !! total protected organic C                 (mass)
                        !! this next term is calculated for C but used for N and Fe as well
                        !! it needs to be protected in case ALL C is protected
                        if (fq4.lt.fq1) then
                           fprotf(ji,jj)   = (fq4 / (fq1 + tiny(fq1)))      !! protected fraction of total organic C     (non-dim)
                        else
                           fprotf(ji,jj)   = 1.0                            !! all organic C is protected                (non-dim)
                        endif
                        fq5      = (1.0 - fprotf(ji,jj))                    !! unprotected fraction of total organic C   (non-dim)
                        fq6      = (fq0 * fq5)                       !! how much organic C is unprotected         (mol)
                        fq7      = (fq6 * exp(-(fse3t(ji,jj,jk) / xfastc)))     !! how much unprotected C leaves this box    (mol)
                        fq8      = (fq7 + (fq0 * fprotf(ji,jj)))            !! how much total C leaves this box          (mol)
                        freminc(ji,jj)  = (fq0 - fq8) / fse3t(ji,jj,jk)                !! C remineralisation in this box            (mol)
                        ffastc(ji,jj) = fq8                          
# if defined key_debug_medusa
                        !! report in/out/remin fluxes of carbon for this level
                           if (idf.eq.1.AND.idfval.eq.1) then
                              IF (lwp) write (numout,*) '------------------------------'
                              IF (lwp) write (numout,*) 'totalC(',jk,')  = ', fq1
                              IF (lwp) write (numout,*) 'prtctC(',jk,')  = ', fq4
                              IF (lwp) write (numout,*) 'fprotf(',jk,')  = ', fprotf(ji,jj)
                              IF (lwp) write (numout,*) '------------------------------'
                              IF (lwp) write (numout,*) 'IN   C(',jk,')  = ', fq0
                              IF (lwp) write (numout,*) 'LOST C(',jk,')  = ', freminc(ji,jj) * fse3t(ji,jj,jk)
                              IF (lwp) write (numout,*) 'OUT  C(',jk,')  = ', fq8
                              IF (lwp) write (numout,*) 'NEW  C(',jk,')  = ', ftempc(ji,jj) * fse3t(ji,jj,jk)
                           endif
# endif
                        else
                        fq1      = fq0 * exp(-(fse3t(ji,jj,jk) / xfastc))       !! how much organic C leaves this box        (mol)
                        freminc(ji,jj)  = (fq0 - fq1) / fse3t(ji,jj,jk)                !! C remineralisation in this box            (mol)
                        ffastc(ji,jj)  = fq1
                     endif
                     !!
                     !! === organic nitrogen ===
                     fq0      = ffastn(ji,jj)                        !! how much organic N enters this box        (mol)
                     if (iball.eq.1) then
                        fq5      = (1.0 - fprotf(ji,jj))                    !! unprotected fraction of total organic N   (non-dim)
                        fq6      = (fq0 * fq5)                       !! how much organic N is unprotected         (mol)
                        fq7      = (fq6 * exp(-(fse3t(ji,jj,jk) / xfastc)))     !! how much unprotected N leaves this box    (mol)
                        fq8      = (fq7 + (fq0 * fprotf(ji,jj)))            !! how much total N leaves this box          (mol)
                        freminn(ji,jj)  = (fq0 - fq8) / fse3t(ji,jj,jk)                !! N remineralisation in this box            (mol)
                        ffastn(ji,jj)  = fq8
# if defined key_debug_medusa
                        !! report in/out/remin fluxes of carbon for this level
                        if (idf.eq.1.AND.idfval.eq.1) then
                           IF (lwp) write (numout,*) '------------------------------'
                           IF (lwp) write (numout,*) 'totalN(',jk,')  = ', fq1
                           IF (lwp) write (numout,*) 'prtctN(',jk,')  = ', fq4
                           IF (lwp) write (numout,*) 'fprotf(',jk,')  = ', fprotf(ji,jj)
                           IF (lwp) write (numout,*) '------------------------------'
                           if (freminn(ji,jj) < 0.0) then
                              IF (lwp) write (numout,*) '** FREMIN ERROR **'
                           endif
                           IF (lwp) write (numout,*) 'IN   N(',jk,')  = ', fq0
                           IF (lwp) write (numout,*) 'LOST N(',jk,')  = ', freminn(ji,jj) * fse3t(ji,jj,jk)
                           IF (lwp) write (numout,*) 'OUT  N(',jk,')  = ', fq8
                           IF (lwp) write (numout,*) 'NEW  N(',jk,')  = ', ftempn(ji,jj) * fse3t(ji,jj,jk)
                        endif
# endif
                     else
                        fq1      = fq0 * exp(-(fse3t(ji,jj,jk) / xfastc))       !! how much organic N leaves this box        (mol)
                        freminn(ji,jj)  = (fq0 - fq1) / fse3t(ji,jj,jk)                !! N remineralisation in this box            (mol)
                        ffastn(ji,jj)  = fq1
                     endif
                     !!
                     !! === organic iron ===
                     fq0      = ffastfe(ji,jj)                       !! how much organic Fe enters this box       (mol)
                     if (iball.eq.1) then
                        fq5      = (1.0 - fprotf(ji,jj))                    !! unprotected fraction of total organic Fe  (non-dim)
                        fq6      = (fq0 * fq5)                       !! how much organic Fe is unprotected        (mol)
                        fq7      = (fq6 * exp(-(fse3t(ji,jj,jk) / xfastc)))     !! how much unprotected Fe leaves this box   (mol)
                        fq8      = (fq7 + (fq0 * fprotf(ji,jj)))            !! how much total Fe leaves this box         (mol)            
                        freminfe(ji,jj) = (fq0 - fq8) / fse3t(ji,jj,jk)                !! Fe remineralisation in this box           (mol)
                        ffastfe(ji,jj) = fq8
                     else
                        fq1      = fq0 * exp(-(fse3t(ji,jj,jk) / xfastc))       !! how much total Fe leaves this box         (mol)      
                        freminfe(ji,jj) = (fq0 - fq1) / fse3t(ji,jj,jk)                !! Fe remineralisation in this box           (mol)
                        ffastfe(ji,jj) = fq1
                     endif
                     !!
                     !! === biogenic silicon ===
                     fq0      = ffastsi(ji,jj)                       !! how much  opal centers this box           (mol) 
                     fq1      = fq0 * exp(-(fse3t(ji,jj,jk) / xfastsi))         !! how much  opal leaves this box            (mol)
                     freminsi(ji,jj) = (fq0 - fq1) / fse3t(ji,jj,jk)                   !! Si remineralisation in this box           (mol)
                     ffastsi(ji,jj) = fq1
                     !!
                     !! === biogenic calcium carbonate ===
                     fq0      = ffastca(ji,jj)                       !! how much CaCO3 enters this box            (mol)
                     if (fsdepw(ji,jj,jk).le.fccd_dep(ji,jj)) then
                        !! whole grid cell above CCD
                        fq1      = fq0                               !! above lysocline, no Ca dissolves          (mol)
                        freminca(ji,jj) = 0.0                               !! above lysocline, no Ca dissolves          (mol)
                        fccd(ji,jj) = real(jk)                       !! which is the last level above the CCD?    (#)
                     elseif (fsdepw(ji,jj,jk).ge.fccd_dep(ji,jj)) then
                        !! whole grid cell below CCD
                        fq1      = fq0 * exp(-(fse3t(ji,jj,jk) / xfastca))      !! how much CaCO3 leaves this box            (mol)
                        freminca(ji,jj) = (fq0 - fq1) / fse3t(ji,jj,jk)                !! Ca remineralisation in this box           (mol)
                     else
                        !! partial grid cell below CCD
                        fq2      = fdep1(ji,jj) - fccd_dep(ji,jj)                  !! amount of grid cell below CCD             (m)
                        fq1      = fq0 * exp(-(fq2 / xfastca))       !! how much CaCO3 leaves this box            (mol)
                        freminca(ji,jj) = (fq0 - fq1) / fse3t(ji,jj,jk)                !! Ca remineralisation in this box           (mol)
                     endif
                     ffastca(ji,jj) = fq1 
                  else
                     !! this is BELOW THE LAST OCEAN BOX (do nothing)
                     freminc(ji,jj)  = 0.0
                     freminn(ji,jj)  = 0.0
                     freminfe(ji,jj) = 0.0
                     freminsi(ji,jj) = 0.0
                     freminca(ji,jj) = 0.0              
                  endif

               elseif (jexport.eq.2.or.jexport.eq.3) then
                  if (jexport.eq.2) then
                     !!======================================================================
                     !! MARTIN ET AL. (1987) SUBMODEL
                     !!======================================================================
                     !! 
                     !!----------------------------------------------------------------------
                     !! This submodel uses the classic Martin et al. (1987) curve
                     !! to determine the attenuation of fast-sinking detritus down
                     !! the water column.  All three organic elements, C, N and Fe,
                     !! are handled identically, and their quantities in sinking
                     !! particles attenuate according to a power relationship
                     !! governed by parameter "b".  This is assigned a canonical 
                     !! value of -0.858.  Biogenic opal and calcium carbonate are
                     !! attentuated using the same function as in the ballast
                     !! submodel
                     !!----------------------------------------------------------------------
                     !!
                     fb_val = -0.858
                  elseif (jexport.eq.3) then
                     !!======================================================================
                     !! HENSON ET AL. (2011) SUBMODEL
                     !!======================================================================
                     !!
                     !!----------------------------------------------------------------------
                     !! This submodel reconfigures the Martin et al. (1987) curve by
                     !! allowing the "b" value to vary geographically.  Its value is
                     !! set, following Henson et al. (2011), as a function of local
                     !! sea surface temperature:
                     !!   b = -1.06 + (0.024 * SST)
                     !! This means that remineralisation length scales are longer in
                     !! warm, tropical areas and shorter in cold, polar areas.  This
                     !! does seem back-to-front (i.e. one would expect GREATER
                     !! remineralisation in warmer waters), but is an outcome of 
                     !! analysis of sediment trap data, and it may reflect details
                     !! of ecosystem structure that pertain to particle production
                     !! rather than simply Q10.
                     !!----------------------------------------------------------------------
                     !!
                     fl_sst = tsn(ji,jj,1,jp_tem)
                     fb_val = -1.06 + (0.024 * fl_sst)
                  endif
                  !!   
                  if (jk.eq.1) then
                     !! this is the SURFACE OCEAN BOX (no remineralisation)
                     !!
                     freminc(ji,jj)  = 0.0
                     freminn(ji,jj)  = 0.0
                     freminfe(ji,jj) = 0.0
                     freminsi(ji,jj) = 0.0
                     freminca(ji,jj) = 0.0
                  elseif (jk.le.mbathy(ji,jj)) then
                     !! this is an OCEAN BOX (remineralise some material)
                     !!
                     !! === organic carbon ===
                     fq0      = ffastc(ji,jj)                        !! how much organic C enters this box        (mol)
                     fq1      = fq0 * ((fdep1(ji,jj)/fsdepw(ji,jj,jk))**fb_val)         !! how much organic C leaves this box        (mol)
                     freminc(ji,jj)  = (fq0 - fq1) / fse3t(ji,jj,jk)                   !! C remineralisation in this box            (mol)
                     ffastc(ji,jj)  = fq1
                     !!
                     !! === organic nitrogen ===
                     fq0      = ffastn(ji,jj)                        !! how much organic N enters this box        (mol)
                     fq1      = fq0 * ((fdep1(ji,jj)/fsdepw(ji,jj,jk))**fb_val)         !! how much organic N leaves this box        (mol)
                     freminn(ji,jj)  = (fq0 - fq1) / fse3t(ji,jj,jk)                   !! N remineralisation in this box            (mol)
                     ffastn(ji,jj)  = fq1
                     !!
                     !! === organic iron ===
                     fq0      = ffastfe(ji,jj)                       !! how much organic Fe enters this box       (mol)
                     fq1      = fq0 * ((fdep1(ji,jj)/fsdepw(ji,jj,jk))**fb_val)         !! how much organic Fe leaves this box       (mol)
                     freminfe(ji,jj) = (fq0 - fq1) / fse3t(ji,jj,jk)                   !! Fe remineralisation in this box           (mol)
                     ffastfe(ji,jj) = fq1
                     !!
                     !! === biogenic silicon ===
                     fq0      = ffastsi(ji,jj)                       !! how much  opal centers this box           (mol) 
                     fq1      = fq0 * exp(-(fse3t(ji,jj,jk) / xfastsi))         !! how much  opal leaves this box            (mol)
                     freminsi(ji,jj) = (fq0 - fq1) / fse3t(ji,jj,jk)                   !! Si remineralisation in this box           (mol)
                     ffastsi(ji,jj) = fq1
                     !!
                     !! === biogenic calcium carbonate ===
                     fq0      = ffastca(ji,jj)                       !! how much CaCO3 enters this box            (mol)
                     if (fsdepw(ji,jj,jk).le.ocal_ccd(ji,jj)) then
                        !! whole grid cell above CCD
                        fq1      = fq0                               !! above lysocline, no Ca dissolves          (mol)
                        freminca(ji,jj) = 0.0                               !! above lysocline, no Ca dissolves          (mol)
                        fccd(ji,jj) = real(jk)                       !! which is the last level above the CCD?    (#)
                     elseif (fsdepw(ji,jj,jk).ge.ocal_ccd(ji,jj)) then
                        !! whole grid cell below CCD
                        fq1      = fq0 * exp(-(fse3t(ji,jj,jk) / xfastca))      !! how much CaCO3 leaves this box            (mol)
                        freminca(ji,jj) = (fq0 - fq1) / fse3t(ji,jj,jk)                !! Ca remineralisation in this box           (mol)
                     else
                        !! partial grid cell below CCD
                        fq2      = fdep1(ji,jj) - ocal_ccd(ji,jj)           !! amount of grid cell below CCD             (m)
                        fq1      = fq0 * exp(-(fq2 / xfastca))       !! how much CaCO3 leaves this box            (mol)
                        freminca(ji,jj) = (fq0 - fq1) / fse3t(ji,jj,jk)                !! Ca remineralisation in this box           (mol)
                     endif
                     ffastca(ji,jj) = fq1 
                  else
                     !! this is BELOW THE LAST OCEAN BOX (do nothing)
                     freminc(ji,jj)  = 0.0
                     freminn(ji,jj)  = 0.0
                     freminfe(ji,jj) = 0.0
                     freminsi(ji,jj) = 0.0
                     freminca(ji,jj) = 0.0              
                  endif

               endif

               !!----------------------------------------------------------------------
               !! Fast-sinking detritus fluxes, pt. 2: UPDATE FAST FLUXES
               !! here locally calculated additions to the fast-sinking flux are added
               !! to the total fast-sinking flux; this is done here such that material
               !! produced in a particular layer is only remineralised below this
               !! layer
               !!----------------------------------------------------------------------
               !!
               !! add sinking material generated in this layer to running totals
               !!
               !! === organic carbon ===                          (diatom and mesozooplankton mortality)
               ffastc(ji,jj)  = ffastc(ji,jj)  + (ftempc(ji,jj)  * fse3t(ji,jj,jk))
               !!
               !! === organic nitrogen ===                        (diatom and mesozooplankton mortality)
               ffastn(ji,jj)  = ffastn(ji,jj)  + (ftempn(ji,jj)  * fse3t(ji,jj,jk))
               !!
               !! === organic iron ===                            (diatom and mesozooplankton mortality)
               ffastfe(ji,jj) = ffastfe(ji,jj) + (ftempfe(ji,jj) * fse3t(ji,jj,jk))
               !!
               !! === biogenic silicon ===                        (diatom mortality and grazed diatoms)
               ffastsi(ji,jj) = ffastsi(ji,jj) + (ftempsi(ji,jj) * fse3t(ji,jj,jk))
               !!
               !! === biogenic calcium carbonate ===              (latitudinally-based fraction of total primary production)
               ffastca(ji,jj) = ffastca(ji,jj) + (ftempca(ji,jj) * fse3t(ji,jj,jk))

               !!----------------------------------------------------------------------
               !! Fast-sinking detritus fluxes, pt. 3: SEAFLOOR
               !! remineralise all remaining fast-sinking detritus to dissolved
               !! nutrients; the sedimentation fluxes calculated here allow the
               !! separation of what's remineralised sinking through the final
               !! ocean box from that which is added to the final box by the
               !! remineralisation of material that reaches the seafloor (i.e.
               !! the model assumes that *all* material that hits the seafloor
               !! is remineralised and that none is permanently buried; hey,
               !! this is a giant GCM model that can't be run for long enough
               !! to deal with burial fluxes!)
               !!
               !! in a change to this process, in part so that MEDUSA behaves
               !! a little more like ERSEM et al., fast-sinking detritus (N, Fe
               !! and C) is converted to slow sinking detritus at the seafloor
               !! instead of being remineralised; the rationale is that in
               !! shallower shelf regions (... that are not fully mixed!) this
               !! allows the detrital material to return slowly to dissolved 
               !! nutrient rather than instantaneously as now; the alternative
               !! would be to explicitly handle seafloor organic material - a
               !! headache I don't wish to experience at this point; note that
               !! fast-sinking Si and Ca detritus is just remineralised as 
               !! per usual
               !! 
               !! AXY (13/01/12)
               !! in a further change to this process, again so that MEDUSA is
               !! a little more like ERSEM et al., material that reaches the
               !! seafloor can now be added to sediment pools and stored for
               !! slow release; there are new 2D arrays for organic nitrogen,
               !! iron and carbon and inorganic silicon and carbon that allow
               !! fast and slow detritus that reaches the seafloor to be held
               !! and released back to the water column more slowly; these arrays
               !! are transferred via the tracer restart files between repeat
               !! submissions of the model
               !!----------------------------------------------------------------------
               !! 
               ffast2slowc(ji,jj)  = 0.0
               ffast2slown(ji,jj)  = 0.0
! I don't think this is used - marc 10/4/17
!               ffast2slowfe(ji,jj) = 0.0
               !!
               if (jk.eq.mbathy(ji,jj)) then
                  !! this is the BOTTOM OCEAN BOX (remineralise everything)
                  !!
                  !! AXY (17/01/12): tweaked to include benthos pools
                  !! 
                  !! === organic carbon ===
                  if (jfdfate.eq.0 .and. jorgben.eq.0) then
                     freminc(ji,jj)  = freminc(ji,jj) + (ffastc(ji,jj) / fse3t(ji,jj,jk))    !! C remineralisation in this box            (mol/m3)
                  elseif (jfdfate.eq.1 .and. jorgben.eq.0) then
                     ffast2slowc(ji,jj) = ffastc(ji,jj) / fse3t(ji,jj,jk)             !! fast C -> slow C                          (mol/m3)
                     fslowc(ji,jj)      = fslowc(ji,jj) + ffast2slowc(ji,jj)
                  elseif (jfdfate.eq.0 .and. jorgben.eq.1) then
                     f_fbenin_c(ji,jj)  = ffastc(ji,jj)             !! fast C -> benthic C                       (mol/m2)
                  endif
                  fsedc(ji,jj)   = ffastc(ji,jj)                          !! record seafloor C                         (mol/m2)
                  ffastc(ji,jj)  = 0.0
                  !!
                  !! === organic nitrogen ===
                  if (jfdfate.eq.0 .and. jorgben.eq.0) then
                     freminn(ji,jj)  = freminn(ji,jj) + (ffastn(ji,jj) / fse3t(ji,jj,jk))    !! N remineralisation in this box            (mol/m3)
                  elseif (jfdfate.eq.1 .and. jorgben.eq.0) then
                     ffast2slown(ji,jj) = ffastn(ji,jj) / fse3t(ji,jj,jk)             !! fast N -> slow N                          (mol/m3)
                     fslown(ji,jj)      = fslown(ji,jj) + ffast2slown(ji,jj)
                  elseif (jfdfate.eq.0 .and. jorgben.eq.1) then
                     f_fbenin_n(ji,jj)  = ffastn(ji,jj)             !! fast N -> benthic N                       (mol/m2)
                  endif
                  fsedn(ji,jj)   = ffastn(ji,jj)                          !! record seafloor N                         (mol/m2)
                  ffastn(ji,jj)  = 0.0
                  !!
                  !! === organic iron ===
                  if (jfdfate.eq.0 .and. jorgben.eq.0) then
                     freminfe(ji,jj) = freminfe(ji,jj) + (ffastfe(ji,jj) / fse3t(ji,jj,jk))  !! Fe remineralisation in this box           (mol/m3)
! I don't think ffast2slowfe is used - marc 10/4/17
!                  elseif (jfdfate.eq.1 .and. jorgben.eq.0) then
!                     ffast2slowfe(ji,jj) = ffastn(ji,jj) / fse3t(ji,jj,jk)            !! fast Fe -> slow Fe                        (mol/m3)
                  elseif (jfdfate.eq.0 .and. jorgben.eq.1) then
                     f_fbenin_fe(ji,jj) = ffastfe(ji,jj)            !! fast Fe -> benthic Fe                     (mol/m2)
                  endif
                  fsedfe(ji,jj)  = ffastfe(ji,jj)                         !! record seafloor Fe                        (mol/m2)
                  ffastfe(ji,jj) = 0.0
                  !!
                  !! === biogenic silicon ===
                  if (jinorgben.eq.0) then
                     freminsi(ji,jj) = freminsi(ji,jj) + (ffastsi(ji,jj) / fse3t(ji,jj,jk))  !! Si remineralisation in this box           (mol/m3)
                  elseif (jinorgben.eq.1) then
                     f_fbenin_si(ji,jj) = ffastsi(ji,jj)            !! fast Si -> benthic Si                     (mol/m2)
                  endif
                  fsedsi(ji,jj)   = ffastsi(ji,jj)                         !! record seafloor Si                        (mol/m2)
                  ffastsi(ji,jj) = 0.0
                  !!
                  !! === biogenic calcium carbonate ===
                  if (jinorgben.eq.0) then
                     freminca(ji,jj) = freminca(ji,jj) + (ffastca(ji,jj) / fse3t(ji,jj,jk))  !! Ca remineralisation in this box           (mol/m3) 
                  elseif (jinorgben.eq.1) then
                     f_fbenin_ca(ji,jj) = ffastca(ji,jj)            !! fast Ca -> benthic Ca                     (mol/m2)
                  endif
                  fsedca(ji,jj)   = ffastca(ji,jj)                         !! record seafloor Ca                        (mol/m2)
                  ffastca(ji,jj) = 0.0
               endif

# if defined key_debug_medusa
               if (idf.eq.1) then
                  !!----------------------------------------------------------------------
                  !! Integrate total fast detritus remineralisation
                  !!----------------------------------------------------------------------
                  !!
                  fofd_n(ji,jj)  = fofd_n(ji,jj)  + (freminn(ji,jj)  * fse3t(ji,jj,jk))
                  fofd_si(ji,jj) = fofd_si(ji,jj) + (freminsi(ji,jj) * fse3t(ji,jj,jk))
                  fofd_fe(ji,jj) = fofd_fe(ji,jj) + (freminfe(ji,jj) * fse3t(ji,jj,jk))
#  if defined key_roam
                  fofd_c(ji,jj)  = fofd_c(ji,jj)  + (freminc(ji,jj)  * fse3t(ji,jj,jk))
#  endif
               endif
# endif

               !!----------------------------------------------------------------------
               !! Sort out remineralisation tally of fast-sinking detritus
               !!----------------------------------------------------------------------
               !!
               !! update fast-sinking regeneration arrays
               fregenfast(ji,jj)   = fregenfast(ji,jj)   + (freminn(ji,jj)  * fse3t(ji,jj,jk))
               fregenfastsi(ji,jj) = fregenfastsi(ji,jj) + (freminsi(ji,jj) * fse3t(ji,jj,jk))
# if defined key_roam
               fregenfastc(ji,jj)  = fregenfastc(ji,jj)  + (freminc(ji,jj)  * fse3t(ji,jj,jk))
# endif

               !!----------------------------------------------------------------------
               !! Benthic remineralisation fluxes
               !!----------------------------------------------------------------------
               !!
               if (jk.eq.mbathy(ji,jj)) then
                  !!
                  !! organic components
                  if (jorgben.eq.1) then
                     f_benout_n(ji,jj)  = xsedn  * zn_sed_n(ji,jj)
                     f_benout_fe(ji,jj) = xsedfe * zn_sed_fe(ji,jj)
                     f_benout_c(ji,jj)  = xsedc  * zn_sed_c(ji,jj)
                  endif
                  !!
                  !! inorganic components
                  if (jinorgben.eq.1) then
                     f_benout_si(ji,jj) = xsedsi * zn_sed_si(ji,jj)
                     f_benout_ca(ji,jj) = xsedca * zn_sed_ca(ji,jj)
                     !!
                     !! account for CaCO3 that dissolves when it shouldn't
                     if ( fsdepw(ji,jj,jk) .le. fccd_dep(ji,jj) ) then
                        f_benout_lyso_ca(ji,jj) = xsedca * zn_sed_ca(ji,jj)
                     endif
                  endif
               endif
               CALL flush(numout)

! MAYBE BUT A BREAK IN HERE, BUSINESS AND UPDATING - marc 20/4/17 
! (fast sinking detritus is 576 lines)
            ENDIF
         ENDDO
         ENDDO

         DO jj = 2,jpjm1
         DO ji = 2,jpim1
            !! OPEN wet point IF..THEN loop
            if (tmask(ji,jj,jk) == 1) then

               !!======================================================================
               !! LOCAL GRID CELL TRENDS
               !!======================================================================
               !!
               !!----------------------------------------------------------------------
               !! Determination of trends
               !!----------------------------------------------------------------------
               !!
               !!----------------------------------------------------------------------
               !! chlorophyll
               btra(ji,jj,jpchn) = b0 * ( &
                 + ((frn(ji,jj) * fprn(ji,jj) * zphn(ji,jj)) - fgmipn(ji,jj) - fgmepn(ji,jj) - fdpn(ji,jj) - fdpn2(ji,jj)) * (fthetan(ji,jj) / xxi) )
               btra(ji,jj,jpchd) = b0 * ( &
                 + ((frd(ji,jj) * fprd(ji,jj) * zphd(ji,jj)) - fgmepd(ji,jj) - fdpd(ji,jj) - fdpd2(ji,jj)) * (fthetad(ji,jj) / xxi) )
               !!
               !!----------------------------------------------------------------------
               !! phytoplankton
               btra(ji,jj,jpphn) = b0 * ( &
                 + (fprn(ji,jj) * zphn(ji,jj)) - fgmipn(ji,jj) - fgmepn(ji,jj) - fdpn(ji,jj) - fdpn2(ji,jj) )
               btra(ji,jj,jpphd) = b0 * ( &
                 + (fprd(ji,jj) * zphd(ji,jj)) - fgmepd(ji,jj) - fdpd(ji,jj) - fdpd2(ji,jj) )
               btra(ji,jj,jppds) = b0 * ( &
                 + (fprds(ji,jj) * zpds(ji,jj)) - fgmepds(ji,jj) - fdpds(ji,jj) - fsdiss(ji,jj) - fdpds2(ji,jj) )
               !!
               !!----------------------------------------------------------------------
               !! zooplankton
               btra(ji,jj,jpzmi) = b0 * ( &
                 + fmigrow(ji,jj) - fgmezmi(ji,jj) - fdzmi(ji,jj) - fdzmi2(ji,jj) )
               btra(ji,jj,jpzme) = b0 * ( &
                 + fmegrow(ji,jj) - fdzme(ji,jj) - fdzme2(ji,jj) )
               !!
               !!----------------------------------------------------------------------
               !! detritus
               btra(ji,jj,jpdet) = b0 * ( &
                 + fdpn(ji,jj) + ((1.0 - xfdfrac1) * fdpd(ji,jj))              &  ! mort. losses
                 + fdzmi(ji,jj) + ((1.0 - xfdfrac2) * fdzme(ji,jj))            &  ! mort. losses
                 + ((1.0 - xbetan) * (finmi(ji,jj) + finme(ji,jj)))            &  ! assim. inefficiency
                 - fgmid(ji,jj) - fgmed(ji,jj) - fdd(ji,jj)                           &  ! grazing and remin.
                 + ffast2slown(ji,jj) )                                    ! seafloor fast->slow
               !!
               !!----------------------------------------------------------------------
               !! dissolved inorganic nitrogen nutrient
               fn_cons = 0.0  &
                 - (fprn(ji,jj) * zphn(ji,jj)) - (fprd(ji,jj) * zphd(ji,jj))                    ! primary production
               fn_prod = 0.0  &
                 + (xphi * (fgmipn(ji,jj) + fgmid(ji,jj)))                     &  ! messy feeding remin.
                 + (xphi * (fgmepn(ji,jj) + fgmepd(ji,jj) + fgmezmi(ji,jj) + fgmed(ji,jj)))  &  ! messy feeding remin.
                 + fmiexcr(ji,jj) + fmeexcr(ji,jj) + fdd(ji,jj) + freminn(ji,jj)             &  ! excretion and remin.
                 + fdpn2(ji,jj) + fdpd2(ji,jj) + fdzmi2(ji,jj) + fdzme2(ji,jj)                  ! metab. losses
               !! 
               !! riverine flux
               if ( jriver_n .gt. 0 ) then
                  f_riv_loc_n(ji,jj) = f_riv_n(ji,jj) * friver_dep(jk,mbathy(ji,jj)) / fse3t(ji,jj,jk)
                  fn_prod = fn_prod + f_riv_loc_n(ji,jj)
               endif
               !!  
               !! benthic remineralisation
               if (jk.eq.mbathy(ji,jj) .and. jorgben.eq.1 .and. ibenthic.eq.1) then
                  fn_prod = fn_prod + (f_benout_n(ji,jj) / fse3t(ji,jj,jk))
               endif
               !!
               btra(ji,jj,jpdin) = b0 * ( &
                 fn_prod + fn_cons )
               !!
               fnit_cons(ji,jj) = fnit_cons(ji,jj) + ( fse3t(ji,jj,jk) * (  &  ! consumption of dissolved nitrogen
                 fn_cons ) )
               fnit_prod(ji,jj) = fnit_prod(ji,jj) + ( fse3t(ji,jj,jk) * (  &  ! production of dissolved nitrogen
                 fn_prod ) )
               !!
               !!----------------------------------------------------------------------
               !! dissolved silicic acid nutrient
               fs_cons = 0.0  &
                 - (fprds(ji,jj) * zpds(ji,jj))                                   ! opal production
               fs_prod = 0.0  &
                 + fsdiss(ji,jj)                                        &  ! opal dissolution
                 + ((1.0 - xfdfrac1) * fdpds(ji,jj))                    &  ! mort. loss
                 + ((1.0 - xfdfrac3) * fgmepds(ji,jj))                  &  ! egestion of grazed Si
                 + freminsi(ji,jj) + fdpds2(ji,jj)                                ! fast diss. and metab. losses
               !! 
               !! riverine flux
               if ( jriver_si .gt. 0 ) then
                  f_riv_loc_si(ji,jj) = f_riv_si(ji,jj) * friver_dep(jk,mbathy(ji,jj)) / fse3t(ji,jj,jk)
                  fs_prod = fs_prod + f_riv_loc_si(ji,jj)
               endif
               !!  
               !! benthic remineralisation
               if (jk.eq.mbathy(ji,jj) .and. jinorgben.eq.1 .and. ibenthic.eq.1) then
                  fs_prod = fs_prod + (f_benout_si(ji,jj) / fse3t(ji,jj,jk))
               endif
               !!
               btra(ji,jj,jpsil) = b0 * ( &
                 fs_prod + fs_cons )
               !!
               fsil_cons(ji,jj) = fsil_cons(ji,jj) + ( fse3t(ji,jj,jk) * (  &  ! consumption of dissolved silicon
                 fs_cons ) )
               fsil_prod(ji,jj) = fsil_prod(ji,jj) + ( fse3t(ji,jj,jk) * (  &  ! production of dissolved silicon
                 fs_prod ) )
               !!
               !!----------------------------------------------------------------------
               !! dissolved "iron" nutrient
               btra(ji,jj,jpfer) = b0 * ( &
               + (xrfn * btra(ji,jj,jpdin)) + ffetop(ji,jj) + ffebot(ji,jj) - ffescav(ji,jj) )

# if defined key_roam
               !!
               !!----------------------------------------------------------------------
               !! AXY (26/11/08): implicit detrital carbon change
               btra(ji,jj,jpdtc) = b0 * ( &
                 + (xthetapn * fdpn(ji,jj)) + ((1.0 - xfdfrac1) * (xthetapd * fdpd(ji,jj)))      &  ! mort. losses
                 + (xthetazmi * fdzmi(ji,jj)) + ((1.0 - xfdfrac2) * (xthetazme * fdzme(ji,jj)))  &  ! mort. losses
                 + ((1.0 - xbetac) * (ficmi(ji,jj) + ficme(ji,jj)))                              &  ! assim. inefficiency
                 - fgmidc(ji,jj) - fgmedc(ji,jj) - fddc(ji,jj)                                          &  ! grazing and remin.
                 + ffast2slowc(ji,jj) )                                                      ! seafloor fast->slow
               !!
               !!----------------------------------------------------------------------
               !! dissolved inorganic carbon
               fc_cons = 0.0  &
                 - (xthetapn * fprn(ji,jj) * zphn(ji,jj)) - (xthetapd * fprd(ji,jj) * zphd(ji,jj))                ! primary production
               fc_prod = 0.0  &
                 + (xthetapn * xphi * fgmipn(ji,jj)) + (xphi * fgmidc(ji,jj))                    &  ! messy feeding remin
                 + (xthetapn * xphi * fgmepn(ji,jj)) + (xthetapd * xphi * fgmepd(ji,jj))         &  ! messy feeding remin
                 + (xthetazmi * xphi * fgmezmi(ji,jj)) + (xphi * fgmedc(ji,jj))                  &  ! messy feeding remin
                 + fmiresp(ji,jj) + fmeresp(ji,jj) + fddc(ji,jj) + freminc(ji,jj) + (xthetapn * fdpn2(ji,jj))         &  ! resp., remin., losses
                 + (xthetapd * fdpd2(ji,jj)) + (xthetazmi * fdzmi2(ji,jj))                       &  ! losses
                 + (xthetazme * fdzme2(ji,jj))                                               ! losses
               !! 
               !! riverine flux
               if ( jriver_c .gt. 0 ) then
                  f_riv_loc_c(ji,jj) = f_riv_c(ji,jj) * friver_dep(jk,mbathy(ji,jj)) / fse3t(ji,jj,jk)
                  fc_prod = fc_prod + f_riv_loc_c(ji,jj)
               endif
               !!  
               !! benthic remineralisation
               if (jk.eq.mbathy(ji,jj) .and. jorgben.eq.1 .and. ibenthic.eq.1) then
                  fc_prod = fc_prod + (f_benout_c(ji,jj) / fse3t(ji,jj,jk))
               endif
               if (jk.eq.mbathy(ji,jj) .and. jinorgben.eq.1 .and. ibenthic.eq.1) then
                  fc_prod = fc_prod + (f_benout_ca(ji,jj) / fse3t(ji,jj,jk))
               endif
               !!
               !! community respiration (does not include CaCO3 terms - obviously!)
               fcomm_resp(ji,jj) = fcomm_resp(ji,jj) + fc_prod
               !!
               !! CaCO3
               fc_prod = fc_prod - ftempca(ji,jj) + freminca(ji,jj)
               !! 
               !! riverine flux
               if ( jk .eq. 1 .and. jriver_c .gt. 0 ) then
                  fc_prod = fc_prod + f_riv_c(ji,jj)
               endif
               !!
               btra(ji,jj,jpdic) = b0 * ( &
                 fc_prod + fc_cons )
               !!
               fcar_cons(ji,jj) = fcar_cons(ji,jj) + ( fse3t(ji,jj,jk) * (  &  ! consumption of dissolved carbon
                 fc_cons ) )
               fcar_prod(ji,jj) = fcar_prod(ji,jj) + ( fse3t(ji,jj,jk) * (  &  ! production of dissolved carbon
                 fc_prod ) )
               !!
               !!----------------------------------------------------------------------
               !! alkalinity
               fa_prod = 0.0  &
                 + (2.0 * freminca(ji,jj))                                                   ! CaCO3 dissolution
               fa_cons = 0.0  &
                 - (2.0 * ftempca(ji,jj))                                                    ! CaCO3 production
               !! 
               !! riverine flux
               if ( jriver_alk .gt. 0 ) then
                  f_riv_loc_alk(ji,jj) = f_riv_alk(ji,jj) * friver_dep(jk,mbathy(ji,jj)) / fse3t(ji,jj,jk)
                  fa_prod = fa_prod + f_riv_loc_alk(ji,jj)
               endif
               !!  
               !! benthic remineralisation
               if (jk.eq.mbathy(ji,jj) .and. jinorgben.eq.1 .and. ibenthic.eq.1) then
                  fa_prod = fa_prod + (2.0 * f_benout_ca(ji,jj) / fse3t(ji,jj,jk))
               endif
               !!
               btra(ji,jj,jpalk) = b0 * ( &
                 fa_prod + fa_cons )
               !!
               !!----------------------------------------------------------------------
               !! oxygen (has protection at low O2 concentrations; OCMIP-2 style)
               fo2_prod(ji,jj) = 0.0 &
                 + (xthetanit * fprn(ji,jj) * zphn(ji,jj))                                      & ! Pn primary production, N
                 + (xthetanit * fprd(ji,jj) * zphd(ji,jj))                                      & ! Pd primary production, N
                 + (xthetarem * xthetapn * fprn(ji,jj) * zphn(ji,jj))                           & ! Pn primary production, C
                 + (xthetarem * xthetapd * fprd(ji,jj) * zphd(ji,jj))                             ! Pd primary production, C
               fo2_ncons(ji,jj) = 0.0 &
                 - (xthetanit * xphi * fgmipn(ji,jj))                                    & ! Pn messy feeding remin., N
                 - (xthetanit * xphi * fgmid(ji,jj))                                     & ! D  messy feeding remin., N
                 - (xthetanit * xphi * fgmepn(ji,jj))                                    & ! Pn messy feeding remin., N
                 - (xthetanit * xphi * fgmepd(ji,jj))                                    & ! Pd messy feeding remin., N
                 - (xthetanit * xphi * fgmezmi(ji,jj))                                   & ! Zi messy feeding remin., N
                 - (xthetanit * xphi * fgmed(ji,jj))                                     & ! D  messy feeding remin., N
                 - (xthetanit * fmiexcr(ji,jj))                                          & ! microzoo excretion, N
                 - (xthetanit * fmeexcr(ji,jj))                                          & ! mesozoo  excretion, N
                 - (xthetanit * fdd(ji,jj))                                              & ! slow detritus remin., N 
                 - (xthetanit * freminn(ji,jj))                                          & ! fast detritus remin., N
                 - (xthetanit * fdpn2(ji,jj))                                            & ! Pn  losses, N
                 - (xthetanit * fdpd2(ji,jj))                                            & ! Pd  losses, N
                 - (xthetanit * fdzmi2(ji,jj))                                           & ! Zmi losses, N
                 - (xthetanit * fdzme2(ji,jj))                                             ! Zme losses, N
               !!  
               !! benthic remineralisation
               if (jk.eq.mbathy(ji,jj) .and. jorgben.eq.1 .and. ibenthic.eq.1) then
                  fo2_ncons(ji,jj) = fo2_ncons(ji,jj) - (xthetanit * f_benout_n(ji,jj) / fse3t(ji,jj,jk))
               endif
               fo2_ccons(ji,jj) = 0.0 &
                 - (xthetarem * xthetapn * xphi * fgmipn(ji,jj))                         & ! Pn messy feeding remin., C
                 - (xthetarem * xphi * fgmidc(ji,jj))                                    & ! D  messy feeding remin., C
                 - (xthetarem * xthetapn * xphi * fgmepn(ji,jj))                         & ! Pn messy feeding remin., C
                 - (xthetarem * xthetapd * xphi * fgmepd(ji,jj))                         & ! Pd messy feeding remin., C
                 - (xthetarem * xthetazmi * xphi * fgmezmi(ji,jj))                       & ! Zi messy feeding remin., C
                 - (xthetarem * xphi * fgmedc(ji,jj))                                    & ! D  messy feeding remin., C
                 - (xthetarem * fmiresp(ji,jj))                                          & ! microzoo respiration, C
                 - (xthetarem * fmeresp(ji,jj))                                          & ! mesozoo  respiration, C
                 - (xthetarem * fddc(ji,jj))                                             & ! slow detritus remin., C
                 - (xthetarem * freminc(ji,jj))                                          & ! fast detritus remin., C
                 - (xthetarem * xthetapn * fdpn2(ji,jj))                                 & ! Pn  losses, C
                 - (xthetarem * xthetapd * fdpd2(ji,jj))                                 & ! Pd  losses, C
                 - (xthetarem * xthetazmi * fdzmi2(ji,jj))                               & ! Zmi losses, C
                 - (xthetarem * xthetazme * fdzme2(ji,jj))                                 ! Zme losses, C
               !!  
               !! benthic remineralisation
               if (jk.eq.mbathy(ji,jj) .and. jorgben.eq.1 .and. ibenthic.eq.1) then
                  fo2_ccons(ji,jj) = fo2_ccons(ji,jj) - (xthetarem * f_benout_c(ji,jj) / fse3t(ji,jj,jk))
               endif
               fo2_cons(ji,jj) = fo2_ncons(ji,jj) + fo2_ccons(ji,jj)
               !!
               !! is this a suboxic zone?
               if (zoxy(ji,jj).lt.xo2min) then  ! deficient O2; production fluxes only
                  btra(ji,jj,jpoxy) = b0 * ( &
                    fo2_prod(ji,jj) )
                  foxy_prod(ji,jj) = foxy_prod(ji,jj) + ( fse3t(ji,jj,jk) * fo2_prod(ji,jj) )
                  foxy_anox(ji,jj) = foxy_anox(ji,jj) + ( fse3t(ji,jj,jk) * fo2_cons(ji,jj) )
               else                      ! sufficient O2; production + consumption fluxes
                  btra(ji,jj,jpoxy) = b0 * ( &
                    fo2_prod(ji,jj) + fo2_cons(ji,jj) )
                  foxy_prod(ji,jj) = foxy_prod(ji,jj) + ( fse3t(ji,jj,jk) * fo2_prod(ji,jj) )
                  foxy_cons(ji,jj) = foxy_cons(ji,jj) + ( fse3t(ji,jj,jk) * fo2_cons(ji,jj) )
               endif
               !!
               !! air-sea fluxes (if this is the surface box)
               if (jk.eq.1) then
                  !!
                  !! CO2 flux
                  btra(ji,jj,jpdic) = btra(ji,jj,jpdic) + (b0 * f_co2flux(ji,jj))
                  !!
                  !! O2 flux (mol/m3/s -> mmol/m3/d)
                  btra(ji,jj,jpoxy) = btra(ji,jj,jpoxy) + (b0 * f_o2flux(ji,jj))
               endif
# endif

# if defined key_debug_medusa
               !! report state variable fluxes (not including fast-sinking detritus)
               if (idf.eq.1.AND.idfval.eq.1) then
                  IF (lwp) write (numout,*) '------------------------------'
                  IF (lwp) write (numout,*) 'btra(ji,jj,jpchn)(',jk,')  = ', btra(ji,jj,jpchn)
                  IF (lwp) write (numout,*) 'btra(ji,jj,jpchd)(',jk,')  = ', btra(ji,jj,jpchd)
                  IF (lwp) write (numout,*) 'btra(ji,jj,jpphn)(',jk,')  = ', btra(ji,jj,jpphn)
                  IF (lwp) write (numout,*) 'btra(ji,jj,jpphd)(',jk,')  = ', btra(ji,jj,jpphd)
                  IF (lwp) write (numout,*) 'btra(ji,jj,jppds)(',jk,')  = ', btra(ji,jj,jppds)
                  IF (lwp) write (numout,*) 'btra(ji,jj,jpzmi)(',jk,')  = ', btra(ji,jj,jpzmi)
                  IF (lwp) write (numout,*) 'btra(ji,jj,jpzme)(',jk,')  = ', btra(ji,jj,jpzme)
                  IF (lwp) write (numout,*) 'btra(ji,jj,jpdet)(',jk,')  = ', btra(ji,jj,jpdet)
                  IF (lwp) write (numout,*) 'btra(ji,jj,jpdin)(',jk,')  = ', btra(ji,jj,jpdin)
                  IF (lwp) write (numout,*) 'btra(ji,jj,jpsil)(',jk,')  = ', btra(ji,jj,jpsil)
                  IF (lwp) write (numout,*) 'btra(ji,jj,jpfer)(',jk,')  = ', btra(ji,jj,jpfer)
#  if defined key_roam
                  IF (lwp) write (numout,*) 'btra(ji,jj,jpdtc)(',jk,')  = ', btra(ji,jj,jpdtc)
                  IF (lwp) write (numout,*) 'btra(ji,jj,jpdic)(',jk,')  = ', btra(ji,jj,jpdic)
                  IF (lwp) write (numout,*) 'btra(ji,jj,jpalk)(',jk,')  = ', btra(ji,jj,jpalk)
                  IF (lwp) write (numout,*) 'btra(ji,jj,jpoxy)(',jk,')  = ', btra(ji,jj,jpoxy)
#  endif
               endif
# endif

               !!----------------------------------------------------------------------
               !! Integrate calculated fluxes for mass balance
               !!----------------------------------------------------------------------
               !!
               !! === nitrogen ===
               fflx_n(ji,jj)  = fflx_n(ji,jj)  + &
                  fse3t(ji,jj,jk) * ( btra(ji,jj,jpphn) + btra(ji,jj,jpphd) + btra(ji,jj,jpzmi) + btra(ji,jj,jpzme) + btra(ji,jj,jpdet) + btra(ji,jj,jpdin) )
               !! === silicon ===
               fflx_si(ji,jj) = fflx_si(ji,jj) + &
                  fse3t(ji,jj,jk) * ( btra(ji,jj,jppds) + btra(ji,jj,jpsil) )
               !! === iron ===
               fflx_fe(ji,jj) = fflx_fe(ji,jj) + &
                  fse3t(ji,jj,jk) * ( ( xrfn * ( btra(ji,jj,jpphn) + btra(ji,jj,jpphd) + btra(ji,jj,jpzmi) + btra(ji,jj,jpzme) + btra(ji,jj,jpdet)) ) + btra(ji,jj,jpfer) )
# if defined key_roam
               !! === carbon ===
               fflx_c(ji,jj)  = fflx_c(ji,jj)  + &
                  fse3t(ji,jj,jk) * ( (xthetapn * btra(ji,jj,jpphn)) + (xthetapd * btra(ji,jj,jpphd)) + &
                  (xthetazmi * btra(ji,jj,jpzmi)) + (xthetazme * btra(ji,jj,jpzme)) + btra(ji,jj,jpdtc) + btra(ji,jj,jpdic) )
               !! === alkalinity ===
               fflx_a(ji,jj)  = fflx_a(ji,jj)  + &
                  fse3t(ji,jj,jk) * ( btra(ji,jj,jpalk) )
               !! === oxygen ===
               fflx_o2(ji,jj) = fflx_o2(ji,jj) + &
                  fse3t(ji,jj,jk) * ( btra(ji,jj,jpoxy) )
# endif

               !!----------------------------------------------------------------------
               !! Apply calculated tracer fluxes
               !!----------------------------------------------------------------------
               !!
               !! units: [unit of tracer] per second (fluxes are calculated above per day)
               !!
               ibio_switch = 1
# if defined key_gulf_finland
               !! AXY (17/05/13): fudge in a Gulf of Finland correction; uses longitude-
               !!                 latitude range to establish if this is a Gulf of Finland 
               !!                 grid cell; if so, then BGC fluxes are ignored (though 
               !!                 still calculated); for reference, this is meant to be a 
               !!                 temporary fix to see if all of my problems can be done 
               !!                 away with if I switch off BGC fluxes in the Gulf of 
               !!                 Finland, which currently appears the source of trouble
               if ( glamt(ji,jj).gt.24.7 .and. glamt(ji,jj).lt.27.8 .and. &
                  &   gphit(ji,jj).gt.59.2 .and. gphit(ji,jj).lt.60.2 ) then
                  ibio_switch = 0
               endif
# endif               
               if (ibio_switch.eq.1) then
                  tra(ji,jj,jk,jpchn) = tra(ji,jj,jk,jpchn) + (btra(ji,jj,jpchn) / 86400.)
                  tra(ji,jj,jk,jpchd) = tra(ji,jj,jk,jpchd) + (btra(ji,jj,jpchd) / 86400.)
                  tra(ji,jj,jk,jpphn) = tra(ji,jj,jk,jpphn) + (btra(ji,jj,jpphn) / 86400.)
                  tra(ji,jj,jk,jpphd) = tra(ji,jj,jk,jpphd) + (btra(ji,jj,jpphd) / 86400.)
                  tra(ji,jj,jk,jppds) = tra(ji,jj,jk,jppds) + (btra(ji,jj,jppds) / 86400.)
                  tra(ji,jj,jk,jpzmi) = tra(ji,jj,jk,jpzmi) + (btra(ji,jj,jpzmi) / 86400.)
                  tra(ji,jj,jk,jpzme) = tra(ji,jj,jk,jpzme) + (btra(ji,jj,jpzme) / 86400.)
                  tra(ji,jj,jk,jpdet) = tra(ji,jj,jk,jpdet) + (btra(ji,jj,jpdet) / 86400.)
                  tra(ji,jj,jk,jpdin) = tra(ji,jj,jk,jpdin) + (btra(ji,jj,jpdin) / 86400.)
                  tra(ji,jj,jk,jpsil) = tra(ji,jj,jk,jpsil) + (btra(ji,jj,jpsil) / 86400.)
                  tra(ji,jj,jk,jpfer) = tra(ji,jj,jk,jpfer) + (btra(ji,jj,jpfer) / 86400.)
# if defined key_roam
                  tra(ji,jj,jk,jpdtc) = tra(ji,jj,jk,jpdtc) + (btra(ji,jj,jpdtc) / 86400.)
                  tra(ji,jj,jk,jpdic) = tra(ji,jj,jk,jpdic) + (btra(ji,jj,jpdic) / 86400.)
                  tra(ji,jj,jk,jpalk) = tra(ji,jj,jk,jpalk) + (btra(ji,jj,jpalk) / 86400.)
                  tra(ji,jj,jk,jpoxy) = tra(ji,jj,jk,jpoxy) + (btra(ji,jj,jpoxy) / 86400.)
# endif
               endif               

               !! AXY (18/11/16): CMIP6 diagnostics
               IF( med_diag%FBDDTALK%dgsave )  THEN
                  fbddtalk(ji,jj)  =  fbddtalk(ji,jj)  + (btra(ji,jj,jpalk) * fse3t(ji,jj,jk))
               ENDIF
               IF( med_diag%FBDDTDIC%dgsave )  THEN
                  fbddtdic(ji,jj)  =  fbddtdic(ji,jj)  + (btra(ji,jj,jpdic) * fse3t(ji,jj,jk))
               ENDIF
               IF( med_diag%FBDDTDIFE%dgsave ) THEN
                  fbddtdife(ji,jj) =  fbddtdife(ji,jj) + (btra(ji,jj,jpfer) * fse3t(ji,jj,jk))
               ENDIF
               IF( med_diag%FBDDTDIN%dgsave )  THEN
                  fbddtdin(ji,jj)  =  fbddtdin(ji,jj)  + (btra(ji,jj,jpdin) * fse3t(ji,jj,jk))
               ENDIF
               IF( med_diag%FBDDTDISI%dgsave ) THEN
                  fbddtdisi(ji,jj) =  fbddtdisi(ji,jj) + (btra(ji,jj,jpsil) * fse3t(ji,jj,jk))
               ENDIF
	       !!
               IF( med_diag%BDDTALK3%dgsave )  THEN
                  bddtalk3(ji,jj,jk)  =  btra(ji,jj,jpalk)
               ENDIF
               IF( med_diag%BDDTDIC3%dgsave )  THEN
                  bddtdic3(ji,jj,jk)  =  btra(ji,jj,jpdic)
               ENDIF
               IF( med_diag%BDDTDIFE3%dgsave ) THEN
                  bddtdife3(ji,jj,jk) =  btra(ji,jj,jpfer)
               ENDIF
               IF( med_diag%BDDTDIN3%dgsave )  THEN
                  bddtdin3(ji,jj,jk)  =  btra(ji,jj,jpdin)
               ENDIF
               IF( med_diag%BDDTDISI3%dgsave ) THEN
                  bddtdisi3(ji,jj,jk) =  btra(ji,jj,jpsil)
               ENDIF

#   if defined key_debug_medusa
               IF (lwp) write (numout,*) '------'
               IF (lwp) write (numout,*) 'trc_bio_medusa: end all calculations'
               IF (lwp) write (numout,*) 'trc_bio_medusa: now outputs'
                     CALL flush(numout)
#   endif

# if defined key_axy_nancheck
               !!----------------------------------------------------------------------
               !! Check calculated tracer fluxes
               !!----------------------------------------------------------------------
               !!
               DO jn = 1,jptra
                  fq0 = btra(ji,jj,jn)
                  !! AXY (30/01/14): "isnan" problem on HECTOR
                  !! if (fq0 /= fq0 ) then
                  if ( ieee_is_nan( fq0 ) ) then
                     !! there's a NaN here
                     if (lwp) write(numout,*) 'NAN detected in btra(ji,jj,', ji, ',', &
                     & jj, ',', jk, ',', jn, ') at time', kt
		     CALL ctl_stop( 'trcbio_medusa, NAN in btra field' )
                  endif
               ENDDO
               DO jn = 1,jptra
                  fq0 = tra(ji,jj,jk,jn)
                  !! AXY (30/01/14): "isnan" problem on HECTOR
                  !! if (fq0 /= fq0 ) then
                  if ( ieee_is_nan( fq0 ) ) then
                     !! there's a NaN here
                     if (lwp) write(numout,*) 'NAN detected in tra(', ji, ',', &
                     & jj, ',', jk, ',', jn, ') at time', kt
   		     CALL ctl_stop( 'trcbio_medusa, NAN in tra field' )
                  endif
               ENDDO
               CALL flush(numout)
# endif

               !!----------------------------------------------------------------------
               !! Check model conservation
               !! these terms merely sum up the tendency terms of the relevant
               !! state variables, which should sum to zero; the iron cycle is
               !! complicated by fluxes that add (aeolian deposition and seafloor
               !! remineralisation) and remove (scavenging) dissolved iron from
               !! the model (i.e. the sum of iron fluxes is unlikely to be zero)
               !!----------------------------------------------------------------------
               !!
               !! fnit0 = btra(ji,jj,jpphn) + btra(ji,jj,jpphd) + btra(ji,jj,jpzmi) + btra(ji,jj,jpzme) + btra(ji,jj,jpdet) + btra(ji,jj,jpdin)  ! + ftempn(ji,jj)
               !! fsil0 = btra(ji,jj,jppds) + btra(ji,jj,jpsil)                              ! + ftempsi(ji,jj)
               !! ffer0 = (xrfn * fnit0) + btra(ji,jj,jpfer)
# if defined key_roam
               !! fcar0 = 0.
               !! falk0 = 0.
               !! foxy0 = 0.
# endif
               !!
               !! if (kt/240*240.eq.kt) then
               !!    if (ji.eq.2.and.jj.eq.2.and.jk.eq.1) then
               !!       IF (lwp) write (*,*) '*******!MEDUSA Conservation!*******',kt
# if defined key_roam
               !!       IF (lwp) write (*,*) fnit0,fsil0,ffer0,fcar0,falk0,foxy0
# else
               !!       IF (lwp) write (*,*) fnit0,fsil0,ffer0
# endif
               !!    endif
               !! endif     

! MAYBE BUT A BREAK IN HERE - marc 20/4/17 
! (this would make the previous section about 470 lines and the one below
! about 700 lines)
            ENDIF
         ENDDO
         ENDDO

         DO jj = 2,jpjm1
         DO ji = 2,jpim1
            !! OPEN wet point IF..THEN loop
            if (tmask(ji,jj,jk) == 1) then

# if defined key_trc_diabio
               !!======================================================================
               !! LOCAL GRID CELL DIAGNOSTICS
               !!======================================================================
               !!
               !!----------------------------------------------------------------------
               !! Full diagnostics key_trc_diabio:
               !! LOBSTER and PISCES support full diagnistics option key_trc_diabio    
               !! which gives an option of FULL output of biological sourses and sinks.
               !! I cannot see any reason for doing this. If needed, it can be done
               !! as shown below.
               !!----------------------------------------------------------------------
               !!
               IF(lwp) WRITE(numout,*) ' MEDUSA does not support key_trc_diabio'
               !!               trbio(ji,jj,jk, 1) = fprn(ji,jj)
# endif

               IF( lk_iomput  .AND.  .NOT.  ln_diatrc  ) THEN
         !!----------------------------------------------------------------------
         !! Add in XML diagnostics stuff
         !!----------------------------------------------------------------------
         !!
         !! ** 2D diagnostics
#   if defined key_debug_medusa
                  IF (lwp) write (numout,*) 'trc_bio_medusa: diag in ij-jj-jk loop'
                  CALL flush(numout)
#   endif
                  IF ( med_diag%PRN%dgsave ) THEN
                      fprn2d(ji,jj) = fprn2d(ji,jj) + (fprn(ji,jj)  * zphn(ji,jj) * fse3t(ji,jj,jk)) 
                  ENDIF
                  IF ( med_diag%MPN%dgsave ) THEN
                      fdpn2d(ji,jj) = fdpn2d(ji,jj) + (fdpn(ji,jj)         * fse3t(ji,jj,jk))
                  ENDIF
                  IF ( med_diag%PRD%dgsave ) THEN
                      fprd2d(ji,jj) = fprd2d(ji,jj) + (fprd(ji,jj)  * zphd(ji,jj) * fse3t(ji,jj,jk))
                  ENDIF
                  IF( med_diag%MPD%dgsave ) THEN
                      fdpd2d(ji,jj) = fdpd2d(ji,jj) + (fdpd(ji,jj)         * fse3t(ji,jj,jk)) 
                  ENDIF
                  !  IF( med_diag%DSED%dgsave ) THEN
                  !      CALL iom_put( "DSED"  , ftot_n )
                  !  ENDIF
                  IF( med_diag%OPAL%dgsave ) THEN
                      fprds2d(ji,jj) = fprds2d(ji,jj) + (fprds(ji,jj) * zpds(ji,jj) * fse3t(ji,jj,jk)) 
                  ENDIF
                  IF( med_diag%OPALDISS%dgsave ) THEN
                      fsdiss2d(ji,jj) = fsdiss2d(ji,jj) + (fsdiss(ji,jj)  * fse3t(ji,jj,jk))  
                  ENDIF
                  IF( med_diag%GMIPn%dgsave ) THEN
                      fgmipn2d(ji,jj) = fgmipn2d(ji,jj) + (fgmipn(ji,jj)  * fse3t(ji,jj,jk)) 
                  ENDIF
                  IF( med_diag%GMID%dgsave ) THEN
                      fgmid2d(ji,jj) = fgmid2d(ji,jj) + (fgmid(ji,jj)   * fse3t(ji,jj,jk)) 
                  ENDIF
                  IF( med_diag%MZMI%dgsave ) THEN
                      fdzmi2d(ji,jj) = fdzmi2d(ji,jj) + (fdzmi(ji,jj)   * fse3t(ji,jj,jk)) 
                  ENDIF
                  IF( med_diag%GMEPN%dgsave ) THEN
                      fgmepn2d(ji,jj) = fgmepn2d(ji,jj) + (fgmepn(ji,jj)  * fse3t(ji,jj,jk))
                  ENDIF
                  IF( med_diag%GMEPD%dgsave ) THEN
                      fgmepd2d(ji,jj) = fgmepd2d(ji,jj) + (fgmepd(ji,jj)  * fse3t(ji,jj,jk)) 
                  ENDIF
                  IF( med_diag%GMEZMI%dgsave ) THEN
                      fgmezmi2d(ji,jj) = fgmezmi2d(ji,jj) + (fgmezmi(ji,jj) * fse3t(ji,jj,jk)) 
                  ENDIF
                  IF( med_diag%GMED%dgsave ) THEN
                      fgmed2d(ji,jj) = fgmed2d(ji,jj) + (fgmed(ji,jj)   * fse3t(ji,jj,jk)) 
                  ENDIF
                  IF( med_diag%MZME%dgsave ) THEN
                      fdzme2d(ji,jj) = fdzme2d(ji,jj) + (fdzme(ji,jj)   * fse3t(ji,jj,jk)) 
                  ENDIF
                  !  IF( med_diag%DEXP%dgsave ) THEN
                  !      CALL iom_put( "DEXP"  , ftot_n )
                  !  ENDIF
                  IF( med_diag%DETN%dgsave ) THEN
                      fslown2d(ji,jj) = fslown2d(ji,jj) + (fslown(ji,jj)  * fse3t(ji,jj,jk))  
                  ENDIF
                  IF( med_diag%MDET%dgsave ) THEN
                      fdd2d(ji,jj) = fdd2d(ji,jj) + (fdd(ji,jj)     * fse3t(ji,jj,jk)) 
                  ENDIF
                  IF( med_diag%AEOLIAN%dgsave ) THEN
                      ffetop2d(ji,jj) = ffetop2d(ji,jj) + (ffetop(ji,jj)  * fse3t(ji,jj,jk)) 
                  ENDIF
                  IF( med_diag%BENTHIC%dgsave ) THEN
                      ffebot2d(ji,jj) = ffebot2d(ji,jj) + (ffebot(ji,jj)  * fse3t(ji,jj,jk)) 
                  ENDIF
                  IF( med_diag%SCAVENGE%dgsave ) THEN
                      ffescav2d(ji,jj) = ffescav2d(ji,jj) + (ffescav(ji,jj) * fse3t(ji,jj,jk))  
                  ENDIF
                  IF( med_diag%PN_JLIM%dgsave ) THEN
                      ! fjln2d(ji,jj) = fjln2d(ji,jj) + (fjln(ji,jj)  * zphn(ji,jj) * fse3t(ji,jj,jk)) 
                      fjln2d(ji,jj) = fjln2d(ji,jj) + (fjlim_pn(ji,jj) * zphn(ji,jj) * fse3t(ji,jj,jk)) 
                  ENDIF
                  IF( med_diag%PN_NLIM%dgsave ) THEN
                      fnln2d(ji,jj) = fnln2d(ji,jj) + (fnln(ji,jj)  * zphn(ji,jj) * fse3t(ji,jj,jk)) 
                  ENDIF
                  IF( med_diag%PN_FELIM%dgsave ) THEN
                      ffln2d(ji,jj) = ffln2d(ji,jj) + (ffln2(ji,jj)  * zphn(ji,jj) * fse3t(ji,jj,jk)) 
                  ENDIF
                  IF( med_diag%PD_JLIM%dgsave ) THEN
                      ! fjld2d(ji,jj) = fjld2d(ji,jj) + (fjld(ji,jj)  * zphd(ji,jj) * fse3t(ji,jj,jk)) 
                      fjld2d(ji,jj) = fjld2d(ji,jj) + (fjlim_pd(ji,jj) * zphd(ji,jj) * fse3t(ji,jj,jk)) 
                  ENDIF
                  IF( med_diag%PD_NLIM%dgsave ) THEN
                      fnld2d(ji,jj) = fnld2d(ji,jj) + (fnld(ji,jj)  * zphd(ji,jj) * fse3t(ji,jj,jk)) 
                  ENDIF
                  IF( med_diag%PD_FELIM%dgsave ) THEN
                      ffld2d(ji,jj) = ffld2d(ji,jj) + (ffld(ji,jj)  * zphd(ji,jj) * fse3t(ji,jj,jk)) 
                  ENDIF
                  IF( med_diag%PD_SILIM%dgsave ) THEN
                      fsld2d2(ji,jj) = fsld2d2(ji,jj) + (fsld2(ji,jj) * zphd(ji,jj) * fse3t(ji,jj,jk)) 
                  ENDIF
                  IF( med_diag%PDSILIM2%dgsave ) THEN
                      fsld2d(ji,jj) = fsld2d(ji,jj) + (fsld(ji,jj)  * zphd(ji,jj) * fse3t(ji,jj,jk))
                  ENDIF
                  !! 
                  IF( med_diag%TOTREG_N%dgsave ) THEN
                      fregen2d(ji,jj) = fregen2d(ji,jj) + fregen(ji,jj)
                  ENDIF
                  IF( med_diag%TOTRG_SI%dgsave ) THEN
                      fregensi2d(ji,jj) = fregensi2d(ji,jj) + fregensi(ji,jj)
                  ENDIF
                  !! 
                  IF( med_diag%FASTN%dgsave ) THEN
                      ftempn2d(ji,jj) = ftempn2d(ji,jj) + (ftempn(ji,jj)  * fse3t(ji,jj,jk))
                  ENDIF
                  IF( med_diag%FASTSI%dgsave ) THEN
                      ftempsi2d(ji,jj) = ftempsi2d(ji,jj) + (ftempsi(ji,jj) * fse3t(ji,jj,jk))
                  ENDIF
                  IF( med_diag%FASTFE%dgsave ) THEN
                      ftempfe2d(ji,jj) =ftempfe2d(ji,jj)  + (ftempfe(ji,jj) * fse3t(ji,jj,jk))  
                  ENDIF
                  IF( med_diag%FASTC%dgsave ) THEN
                      ftempc2d(ji,jj) = ftempc2d(ji,jj) + (ftempc(ji,jj)  * fse3t(ji,jj,jk))
                  ENDIF
                  IF( med_diag%FASTCA%dgsave ) THEN
                      ftempca2d(ji,jj) = ftempca2d(ji,jj) + (ftempca(ji,jj) * fse3t(ji,jj,jk))
                  ENDIF
                  !! 
                  IF( med_diag%REMINN%dgsave ) THEN
                      freminn2d(ji,jj) = freminn2d(ji,jj) + (freminn(ji,jj)  * fse3t(ji,jj,jk))
                  ENDIF
                  IF( med_diag%REMINSI%dgsave ) THEN
                      freminsi2d(ji,jj) = freminsi2d(ji,jj) + (freminsi(ji,jj) * fse3t(ji,jj,jk))
                  ENDIF
                  IF( med_diag%REMINFE%dgsave ) THEN
                      freminfe2d(ji,jj)= freminfe2d(ji,jj) + (freminfe(ji,jj) * fse3t(ji,jj,jk)) 
                  ENDIF
                  IF( med_diag%REMINC%dgsave ) THEN
                      freminc2d(ji,jj) = freminc2d(ji,jj) + (freminc(ji,jj)  * fse3t(ji,jj,jk)) 
                  ENDIF
                  IF( med_diag%REMINCA%dgsave ) THEN
                      freminca2d(ji,jj) = freminca2d(ji,jj) + (freminca(ji,jj) * fse3t(ji,jj,jk)) 
                  ENDIF
                  !!
# if defined key_roam
                  !!
                  !! AXY (09/11/16): CMIP6 diagnostics
                  IF( med_diag%FD_NIT3%dgsave ) THEN
                     fd_nit3(ji,jj,jk) = ffastn(ji,jj)
                  ENDIF
                  IF( med_diag%FD_SIL3%dgsave ) THEN
                     fd_sil3(ji,jj,jk) = ffastsi(ji,jj)
                  ENDIF
                  IF( med_diag%FD_CAR3%dgsave ) THEN
                     fd_car3(ji,jj,jk) = ffastc(ji,jj)
                  ENDIF
                  IF( med_diag%FD_CAL3%dgsave ) THEN
                     fd_cal3(ji,jj,jk) = ffastca(ji,jj)
                  ENDIF
                  !!
                  IF (jk.eq.i0100) THEN
                     IF( med_diag%RR_0100%dgsave ) THEN
                        ffastca2d(ji,jj) =   &
                        ffastca(ji,jj)/MAX(ffastc(ji,jj), rsmall)
                     ENDIF                     
                  ELSE IF (jk.eq.i0500) THEN 
                     IF( med_diag%RR_0500%dgsave ) THEN
                        ffastca2d(ji,jj) =   &
                        ffastca(ji,jj)/MAX(ffastc(ji,jj), rsmall)
                     ENDIF                        
                  ELSE IF (jk.eq.i1000) THEN
                     IF( med_diag%RR_1000%dgsave ) THEN
                        ffastca2d(ji,jj) =   &
                        ffastca(ji,jj)/MAX(ffastc(ji,jj), rsmall)
                     ENDIF
                  ELSE IF (jk.eq.mbathy(ji,jj)) THEN
                     IF( med_diag%IBEN_N%dgsave ) THEN
                        iben_n2d(ji,jj) = f_sbenin_n(ji,jj)  + f_fbenin_n(ji,jj)
                     ENDIF
                     IF( med_diag%IBEN_FE%dgsave ) THEN
                        iben_fe2d(ji,jj) = f_sbenin_fe(ji,jj) + f_fbenin_fe(ji,jj)
                     ENDIF
                     IF( med_diag%IBEN_C%dgsave ) THEN
                        iben_c2d(ji,jj) = f_sbenin_c(ji,jj)  + f_fbenin_c(ji,jj)
                     ENDIF
                     IF( med_diag%IBEN_SI%dgsave ) THEN
                        iben_si2d(ji,jj) = f_fbenin_si(ji,jj)
                     ENDIF
                     IF( med_diag%IBEN_CA%dgsave ) THEN
                        iben_ca2d(ji,jj) = f_fbenin_ca(ji,jj)
                     ENDIF
                     IF( med_diag%OBEN_N%dgsave ) THEN
                        oben_n2d(ji,jj) = f_benout_n(ji,jj)
                     ENDIF
                     IF( med_diag%OBEN_FE%dgsave ) THEN
                        oben_fe2d(ji,jj) = f_benout_fe(ji,jj)
                     ENDIF
                     IF( med_diag%OBEN_C%dgsave ) THEN
                        oben_c2d(ji,jj) = f_benout_c(ji,jj)
                     ENDIF
                     IF( med_diag%OBEN_SI%dgsave ) THEN
                        oben_si2d(ji,jj) = f_benout_si(ji,jj)
                     ENDIF
                     IF( med_diag%OBEN_CA%dgsave ) THEN
                        oben_ca2d(ji,jj) = f_benout_ca(ji,jj)
                     ENDIF
                     IF( med_diag%SFR_OCAL%dgsave ) THEN
                        sfr_ocal2d(ji,jj) = f3_omcal(ji,jj,jk)
                     ENDIF
                     IF( med_diag%SFR_OARG%dgsave ) THEN
                        sfr_oarg2d(ji,jj) =  f3_omarg(ji,jj,jk)
                     ENDIF
                     IF( med_diag%LYSO_CA%dgsave ) THEN
                        lyso_ca2d(ji,jj) = f_benout_lyso_ca(ji,jj)
                     ENDIF
                  ENDIF
                  !! end bathy-1 diags
                  !!
                  IF( med_diag%RIV_N%dgsave ) THEN
                     rivn2d(ji,jj) = rivn2d(ji,jj) +  (f_riv_loc_n(ji,jj) * fse3t(ji,jj,jk))
                  ENDIF
                  IF( med_diag%RIV_SI%dgsave ) THEN
                     rivsi2d(ji,jj) = rivsi2d(ji,jj) +  (f_riv_loc_si(ji,jj) * fse3t(ji,jj,jk))
                  ENDIF
                  IF( med_diag%RIV_C%dgsave ) THEN
                     rivc2d(ji,jj) = rivc2d(ji,jj) +  (f_riv_loc_c(ji,jj) * fse3t(ji,jj,jk))
                  ENDIF
                  IF( med_diag%RIV_ALK%dgsave ) THEN
                     rivalk2d(ji,jj) = rivalk2d(ji,jj) +  (f_riv_loc_alk(ji,jj) * fse3t(ji,jj,jk))
                  ENDIF
                  IF( med_diag%DETC%dgsave ) THEN
                     fslowc2d(ji,jj) = fslowc2d(ji,jj) + (fslowc(ji,jj)  * fse3t(ji,jj,jk))   
                  ENDIF
                  !! 
                  !!              
                  !!
                  IF( med_diag%PN_LLOSS%dgsave ) THEN
                     fdpn22d(ji,jj) = fdpn22d(ji,jj) + (fdpn2(ji,jj)  * fse3t(ji,jj,jk))
                  ENDIF
                  IF( med_diag%PD_LLOSS%dgsave ) THEN
                     fdpd22d(ji,jj) = fdpd22d(ji,jj) + (fdpd2(ji,jj)  * fse3t(ji,jj,jk))
                  ENDIF
                  IF( med_diag%ZI_LLOSS%dgsave ) THEN
                     fdzmi22d(ji,jj) = fdzmi22d(ji,jj) + (fdzmi2(ji,jj) * fse3t(ji,jj,jk))
                  ENDIF
                  IF( med_diag%ZE_LLOSS%dgsave ) THEN
                     fdzme22d(ji,jj) = fdzme22d(ji,jj) + (fdzme2(ji,jj) * fse3t(ji,jj,jk))
                  ENDIF
                  IF( med_diag%ZI_MES_N%dgsave ) THEN
                     zimesn2d(ji,jj) = zimesn2d(ji,jj) +  &
                     (xphi * (fgmipn(ji,jj) + fgmid(ji,jj)) * fse3t(ji,jj,jk))
                  ENDIF
                  IF( med_diag%ZI_MES_D%dgsave ) THEN
                     zimesd2d(ji,jj) = zimesd2d(ji,jj) + & 
                     ((1. - xbetan) * finmi(ji,jj) * fse3t(ji,jj,jk))
                  ENDIF
                  IF( med_diag%ZI_MES_C%dgsave ) THEN
                     zimesc2d(ji,jj) = zimesc2d(ji,jj) + &
                     (xphi * ((xthetapn * fgmipn(ji,jj)) + fgmidc(ji,jj)) * fse3t(ji,jj,jk))
                  ENDIF
                  IF( med_diag%ZI_MESDC%dgsave ) THEN
                     zimesdc2d(ji,jj) = zimesdc2d(ji,jj) + &
                     ((1. - xbetac) * ficmi(ji,jj) * fse3t(ji,jj,jk))
                  ENDIF
                  IF( med_diag%ZI_EXCR%dgsave ) THEN
                     ziexcr2d(ji,jj) = ziexcr2d(ji,jj) +  (fmiexcr(ji,jj) * fse3t(ji,jj,jk))
                  ENDIF
                  IF( med_diag%ZI_RESP%dgsave ) THEN
                     ziresp2d(ji,jj) = ziresp2d(ji,jj) +  (fmiresp(ji,jj) * fse3t(ji,jj,jk))
                  ENDIF
                  IF( med_diag%ZI_GROW%dgsave ) THEN
                     zigrow2d(ji,jj) = zigrow2d(ji,jj) + (fmigrow(ji,jj) * fse3t(ji,jj,jk))
                  ENDIF
                  IF( med_diag%ZE_MES_N%dgsave ) THEN
                     zemesn2d(ji,jj) = zemesn2d(ji,jj) + &
                     (xphi * (fgmepn(ji,jj) + fgmepd(ji,jj) + fgmezmi(ji,jj) + fgmed(ji,jj)) * fse3t(ji,jj,jk))
                  ENDIF
                  IF( med_diag%ZE_MES_D%dgsave ) THEN
                     zemesd2d(ji,jj) = zemesd2d(ji,jj) + &
                     ((1. - xbetan) * finme(ji,jj) * fse3t(ji,jj,jk))
                  ENDIF
                  IF( med_diag%ZE_MES_C%dgsave ) THEN
                     zemesc2d(ji,jj) = zemesc2d(ji,jj) +                         & 
                     (xphi * ((xthetapn * fgmepn(ji,jj)) + (xthetapd * fgmepd(ji,jj)) +  &
                     (xthetazmi * fgmezmi(ji,jj)) + fgmedc(ji,jj)) * fse3t(ji,jj,jk))
                  ENDIF
                  IF( med_diag%ZE_MESDC%dgsave ) THEN
                     zemesdc2d(ji,jj) = zemesdc2d(ji,jj) +  &
                     ((1. - xbetac) * ficme(ji,jj) * fse3t(ji,jj,jk))
                  ENDIF
                  IF( med_diag%ZE_EXCR%dgsave ) THEN
                     zeexcr2d(ji,jj) = zeexcr2d(ji,jj) + (fmeexcr(ji,jj) * fse3t(ji,jj,jk))
                  ENDIF
                  IF( med_diag%ZE_RESP%dgsave ) THEN
                     zeresp2d(ji,jj) = zeresp2d(ji,jj) + (fmeresp(ji,jj) * fse3t(ji,jj,jk))
                  ENDIF
                  IF( med_diag%ZE_GROW%dgsave ) THEN
                     zegrow2d(ji,jj) = zegrow2d(ji,jj) + (fmegrow(ji,jj) * fse3t(ji,jj,jk))
                  ENDIF
                  IF( med_diag%MDETC%dgsave ) THEN
                     mdetc2d(ji,jj) = mdetc2d(ji,jj) + (fddc(ji,jj) * fse3t(ji,jj,jk))
                  ENDIF
                  IF( med_diag%GMIDC%dgsave ) THEN
                     gmidc2d(ji,jj) = gmidc2d(ji,jj) + (fgmidc(ji,jj) * fse3t(ji,jj,jk))
                  ENDIF
                  IF( med_diag%GMEDC%dgsave ) THEN
                     gmedc2d(ji,jj) = gmedc2d(ji,jj) + (fgmedc(ji,jj)  * fse3t(ji,jj,jk))
                  ENDIF
                  !!
# endif                   
                  !!
                  !! ** 3D diagnostics
                  IF( med_diag%TPP3%dgsave ) THEN
                     tpp3d(ji,jj,jk) =  (fprn(ji,jj) * zphn(ji,jj)) + (fprd(ji,jj) * zphd(ji,jj))
                     !CALL iom_put( "TPP3"  , tpp3d )
                  ENDIF
                  IF( med_diag%TPPD3%dgsave ) THEN
                     tppd3(ji,jj,jk) =  (fprd(ji,jj) * zphd(ji,jj))
                  ENDIF
                  
                  IF( med_diag%REMIN3N%dgsave ) THEN
                     remin3dn(ji,jj,jk) = fregen(ji,jj) + (freminn(ji,jj) * fse3t(ji,jj,jk)) !! remineralisation
                     !CALL iom_put( "REMIN3N"  , remin3dn )
                  ENDIF
                  !! IF( med_diag%PH3%dgsave ) THEN
                  !!     CALL iom_put( "PH3"  , f3_pH )
                  !! ENDIF
                  !! IF( med_diag%OM_CAL3%dgsave ) THEN
                  !!     CALL iom_put( "OM_CAL3"  , f3_omcal )
                  !! ENDIF
		  !! 
		  !! AXY (09/11/16): CMIP6 diagnostics
		  IF ( med_diag%DCALC3%dgsave   ) THEN
                     dcalc3(ji,jj,jk) = freminca(ji,jj)
                  ENDIF
		  IF ( med_diag%FEDISS3%dgsave  ) THEN
                     fediss3(ji,jj,jk) = ffetop(ji,jj)
                  ENDIF
		  IF ( med_diag%FESCAV3%dgsave  ) THEN
                     fescav3(ji,jj,jk) = ffescav(ji,jj)
                  ENDIF
		  IF ( med_diag%MIGRAZP3%dgsave ) THEN
                     migrazp3(ji,jj,jk) = fgmipn(ji,jj) * xthetapn
                  ENDIF
		  IF ( med_diag%MIGRAZD3%dgsave ) THEN
                     migrazd3(ji,jj,jk) = fgmidc(ji,jj)
                  ENDIF
		  IF ( med_diag%MEGRAZP3%dgsave ) THEN
                     megrazp3(ji,jj,jk) = (fgmepn(ji,jj) * xthetapn) + (fgmepd(ji,jj) * xthetapd)
                  ENDIF
		  IF ( med_diag%MEGRAZD3%dgsave ) THEN
                     megrazd3(ji,jj,jk) = fgmedc(ji,jj)
                  ENDIF
		  IF ( med_diag%MEGRAZZ3%dgsave ) THEN
                     megrazz3(ji,jj,jk) = (fgmezmi(ji,jj) * xthetazmi)
                  ENDIF
		  IF ( med_diag%PBSI3%dgsave    ) THEN
                     pbsi3(ji,jj,jk)    = (fprds(ji,jj) * zpds(ji,jj))
                  ENDIF
		  IF ( med_diag%PCAL3%dgsave    ) THEN
                     pcal3(ji,jj,jk)    = ftempca(ji,jj)
                  ENDIF
		  IF ( med_diag%REMOC3%dgsave   ) THEN
                     remoc3(ji,jj,jk)   = freminc(ji,jj)
                  ENDIF
		  IF ( med_diag%PNLIMJ3%dgsave  ) THEN
                     ! pnlimj3(ji,jj,jk)  = fjln(ji,jj)
                     pnlimj3(ji,jj,jk)  = fjlim_pn(ji,jj)
                  ENDIF
		  IF ( med_diag%PNLIMN3%dgsave  ) THEN
                     pnlimn3(ji,jj,jk)  = fnln(ji,jj)
                  ENDIF
		  IF ( med_diag%PNLIMFE3%dgsave ) THEN
                     pnlimfe3(ji,jj,jk) = ffln2(ji,jj)
                  ENDIF
		  IF ( med_diag%PDLIMJ3%dgsave  ) THEN
                     ! pdlimj3(ji,jj,jk)  = fjld(ji,jj)
                     pdlimj3(ji,jj,jk)  = fjlim_pd(ji,jj)
                  ENDIF
		  IF ( med_diag%PDLIMN3%dgsave  ) THEN
                     pdlimn3(ji,jj,jk)  = fnld(ji,jj)
                  ENDIF
		  IF ( med_diag%PDLIMFE3%dgsave ) THEN
                     pdlimfe3(ji,jj,jk) = ffld(ji,jj)
                  ENDIF
		  IF ( med_diag%PDLIMSI3%dgsave ) THEN
                     pdlimsi3(ji,jj,jk) = fsld2(ji,jj)
                  ENDIF
                  !!
                  !! ** Without using iom_use
               ELSE IF( ln_diatrc ) THEN
#   if defined key_debug_medusa
                  IF (lwp) write (numout,*) 'trc_bio_medusa: diag in ij-jj-jk ln_diatrc'
                  CALL flush(numout)
#   endif
                  !!----------------------------------------------------------------------
                  !! Prepare 2D diagnostics
                  !!----------------------------------------------------------------------
                  !!
                  !! if ((kt / 240*240).eq.kt) then
                  !!    IF (lwp) write (*,*) '*******!MEDUSA DIAADD!*******',kt
                  !! endif     
                  trc2d(ji,jj,1)  =  ftot_n(ji,jj)                             !! nitrogen inventory
                  trc2d(ji,jj,2)  =  ftot_si(ji,jj)                            !! silicon  inventory
                  trc2d(ji,jj,3)  =  ftot_fe(ji,jj)                            !! iron     inventory
                  trc2d(ji,jj,4)  = trc2d(ji,jj,4)  + (fprn(ji,jj)  * zphn(ji,jj) * fse3t(ji,jj,jk))    !! non-diatom production
                  trc2d(ji,jj,5)  = trc2d(ji,jj,5)  + (fdpn(ji,jj)         * fse3t(ji,jj,jk))    !! non-diatom non-grazing losses
                  trc2d(ji,jj,6)  = trc2d(ji,jj,6)  + (fprd(ji,jj)  * zphd(ji,jj) * fse3t(ji,jj,jk))    !! diatom production
                  trc2d(ji,jj,7)  = trc2d(ji,jj,7)  + (fdpd(ji,jj)         * fse3t(ji,jj,jk))    !! diatom non-grazing losses
                  !! diagnostic field  8 is (ostensibly) supplied by trcsed.F            
                  trc2d(ji,jj,9)  = trc2d(ji,jj,9)  + (fprds(ji,jj) * zpds(ji,jj) * fse3t(ji,jj,jk))    !! diatom silicon production
                  trc2d(ji,jj,10) = trc2d(ji,jj,10) + (fsdiss(ji,jj)  * fse3t(ji,jj,jk))         !! diatom silicon dissolution
                  trc2d(ji,jj,11) = trc2d(ji,jj,11) + (fgmipn(ji,jj)  * fse3t(ji,jj,jk))         !! microzoo grazing on non-diatoms
                  trc2d(ji,jj,12) = trc2d(ji,jj,12) + (fgmid(ji,jj)   * fse3t(ji,jj,jk))         !! microzoo grazing on detrital nitrogen
                  trc2d(ji,jj,13) = trc2d(ji,jj,13) + (fdzmi(ji,jj)   * fse3t(ji,jj,jk))         !! microzoo non-grazing losses
                  trc2d(ji,jj,14) = trc2d(ji,jj,14) + (fgmepn(ji,jj)  * fse3t(ji,jj,jk))         !! mesozoo  grazing on non-diatoms
                  trc2d(ji,jj,15) = trc2d(ji,jj,15) + (fgmepd(ji,jj)  * fse3t(ji,jj,jk))         !! mesozoo  grazing on diatoms
                  trc2d(ji,jj,16) = trc2d(ji,jj,16) + (fgmezmi(ji,jj) * fse3t(ji,jj,jk))         !! mesozoo  grazing on microzoo
                  trc2d(ji,jj,17) = trc2d(ji,jj,17) + (fgmed(ji,jj)   * fse3t(ji,jj,jk))         !! mesozoo  grazing on detrital nitrogen
                  trc2d(ji,jj,18) = trc2d(ji,jj,18) + (fdzme(ji,jj)   * fse3t(ji,jj,jk))         !! mesozoo  non-grazing losses
                  !! diagnostic field 19 is (ostensibly) supplied by trcexp.F
                  trc2d(ji,jj,20) = trc2d(ji,jj,20) + (fslown(ji,jj)  * fse3t(ji,jj,jk))         !! slow sinking detritus N production
                  trc2d(ji,jj,21) = trc2d(ji,jj,21) + (fdd(ji,jj)     * fse3t(ji,jj,jk))         !! detrital remineralisation
                  trc2d(ji,jj,22) = trc2d(ji,jj,22) + (ffetop(ji,jj)  * fse3t(ji,jj,jk))         !! aeolian  iron addition
                  trc2d(ji,jj,23) = trc2d(ji,jj,23) + (ffebot(ji,jj)  * fse3t(ji,jj,jk))         !! seafloor iron addition
                  trc2d(ji,jj,24) = trc2d(ji,jj,24) + (ffescav(ji,jj) * fse3t(ji,jj,jk))         !! "free" iron scavenging
                  trc2d(ji,jj,25) = trc2d(ji,jj,25) + (fjlim_pn(ji,jj) * zphn(ji,jj) * fse3t(ji,jj,jk)) !! non-diatom J  limitation term 
                  trc2d(ji,jj,26) = trc2d(ji,jj,26) + (fnln(ji,jj)  * zphn(ji,jj) * fse3t(ji,jj,jk))    !! non-diatom N  limitation term 
                  trc2d(ji,jj,27) = trc2d(ji,jj,27) + (ffln2(ji,jj)  * zphn(ji,jj) * fse3t(ji,jj,jk))    !! non-diatom Fe limitation term 
                  trc2d(ji,jj,28) = trc2d(ji,jj,28) + (fjlim_pd(ji,jj) * zphd(ji,jj) * fse3t(ji,jj,jk)) !! diatom     J  limitation term 
                  trc2d(ji,jj,29) = trc2d(ji,jj,29) + (fnld(ji,jj)  * zphd(ji,jj) * fse3t(ji,jj,jk))    !! diatom     N  limitation term 
                  trc2d(ji,jj,30) = trc2d(ji,jj,30) + (ffld(ji,jj)  * zphd(ji,jj) * fse3t(ji,jj,jk))    !! diatom     Fe limitation term 
                  trc2d(ji,jj,31) = trc2d(ji,jj,31) + (fsld2(ji,jj) * zphd(ji,jj) * fse3t(ji,jj,jk))    !! diatom     Si limitation term 
                  trc2d(ji,jj,32) = trc2d(ji,jj,32) + (fsld(ji,jj)  * zphd(ji,jj) * fse3t(ji,jj,jk))    !! diatom     Si uptake limitation term
                  if (jk.eq.i0100) trc2d(ji,jj,33) = fslownflux(ji,jj)         !! slow detritus flux at  100 m
                  if (jk.eq.i0200) trc2d(ji,jj,34) = fslownflux(ji,jj)         !! slow detritus flux at  200 m
                  if (jk.eq.i0500) trc2d(ji,jj,35) = fslownflux(ji,jj)         !! slow detritus flux at  500 m
                  if (jk.eq.i1000) trc2d(ji,jj,36) = fslownflux(ji,jj)         !! slow detritus flux at 1000 m
                  trc2d(ji,jj,37) = trc2d(ji,jj,37) + fregen(ji,jj)                   !! non-fast N  full column regeneration
                  trc2d(ji,jj,38) = trc2d(ji,jj,38) + fregensi(ji,jj)                 !! non-fast Si full column regeneration
                  if (jk.eq.i0100) trc2d(ji,jj,39) = trc2d(ji,jj,37)           !! non-fast N  regeneration to  100 m
                  if (jk.eq.i0200) trc2d(ji,jj,40) = trc2d(ji,jj,37)           !! non-fast N  regeneration to  200 m
                  if (jk.eq.i0500) trc2d(ji,jj,41) = trc2d(ji,jj,37)           !! non-fast N  regeneration to  500 m
                  if (jk.eq.i1000) trc2d(ji,jj,42) = trc2d(ji,jj,37)           !! non-fast N  regeneration to 1000 m
                  trc2d(ji,jj,43) = trc2d(ji,jj,43) + (ftempn(ji,jj)  * fse3t(ji,jj,jk))         !! fast sinking detritus N production
                  trc2d(ji,jj,44) = trc2d(ji,jj,44) + (ftempsi(ji,jj) * fse3t(ji,jj,jk))         !! fast sinking detritus Si production
                  trc2d(ji,jj,45) = trc2d(ji,jj,45) + (ftempfe(ji,jj) * fse3t(ji,jj,jk))         !! fast sinking detritus Fe production
                  trc2d(ji,jj,46) = trc2d(ji,jj,46) + (ftempc(ji,jj)  * fse3t(ji,jj,jk))         !! fast sinking detritus C production
                  trc2d(ji,jj,47) = trc2d(ji,jj,47) + (ftempca(ji,jj) * fse3t(ji,jj,jk))         !! fast sinking detritus CaCO3 production
                  if (jk.eq.i0100) trc2d(ji,jj,48) = ffastn(ji,jj)             !! fast detritus N  flux at  100 m
                  if (jk.eq.i0200) trc2d(ji,jj,49) = ffastn(ji,jj)             !! fast detritus N  flux at  200 m
                  if (jk.eq.i0500) trc2d(ji,jj,50) = ffastn(ji,jj)             !! fast detritus N  flux at  500 m
                  if (jk.eq.i1000) trc2d(ji,jj,51) = ffastn(ji,jj)             !! fast detritus N  flux at 1000 m
                  if (jk.eq.i0100) trc2d(ji,jj,52) = fregenfast(ji,jj)         !! N  regeneration to  100 m
                  if (jk.eq.i0200) trc2d(ji,jj,53) = fregenfast(ji,jj)         !! N  regeneration to  200 m
                  if (jk.eq.i0500) trc2d(ji,jj,54) = fregenfast(ji,jj)         !! N  regeneration to  500 m
                  if (jk.eq.i1000) trc2d(ji,jj,55) = fregenfast(ji,jj)         !! N  regeneration to 1000 m
                  if (jk.eq.i0100) trc2d(ji,jj,56) = ffastsi(ji,jj)            !! fast detritus Si flux at  100 m
                  if (jk.eq.i0200) trc2d(ji,jj,57) = ffastsi(ji,jj)            !! fast detritus Si flux at  200 m
                  if (jk.eq.i0500) trc2d(ji,jj,58) = ffastsi(ji,jj)            !! fast detritus Si flux at  500 m
                  if (jk.eq.i1000) trc2d(ji,jj,59) = ffastsi(ji,jj)            !! fast detritus Si flux at 1000 m
                  if (jk.eq.i0100) trc2d(ji,jj,60) = fregenfastsi(ji,jj)       !! Si regeneration to  100 m
                  if (jk.eq.i0200) trc2d(ji,jj,61) = fregenfastsi(ji,jj)       !! Si regeneration to  200 m
                  if (jk.eq.i0500) trc2d(ji,jj,62) = fregenfastsi(ji,jj)       !! Si regeneration to  500 m
                  if (jk.eq.i1000) trc2d(ji,jj,63) = fregenfastsi(ji,jj)       !! Si regeneration to 1000 m
                  trc2d(ji,jj,64) = trc2d(ji,jj,64) + (freminn(ji,jj)  * fse3t(ji,jj,jk))        !! sum of fast-sinking N  fluxes
                  trc2d(ji,jj,65) = trc2d(ji,jj,65) + (freminsi(ji,jj) * fse3t(ji,jj,jk))        !! sum of fast-sinking Si fluxes
                  trc2d(ji,jj,66) = trc2d(ji,jj,66) + (freminfe(ji,jj) * fse3t(ji,jj,jk))        !! sum of fast-sinking Fe fluxes
                  trc2d(ji,jj,67) = trc2d(ji,jj,67) + (freminc(ji,jj)  * fse3t(ji,jj,jk))        !! sum of fast-sinking C  fluxes
                  trc2d(ji,jj,68) = trc2d(ji,jj,68) + (freminca(ji,jj) * fse3t(ji,jj,jk))        !! sum of fast-sinking Ca fluxes
                  if (jk.eq.mbathy(ji,jj)) then
                     trc2d(ji,jj,69) = fsedn(ji,jj)                                   !! N  sedimentation flux                                  
                     trc2d(ji,jj,70) = fsedsi(ji,jj)                                  !! Si sedimentation flux
                     trc2d(ji,jj,71) = fsedfe(ji,jj)                                  !! Fe sedimentation flux
                     trc2d(ji,jj,72) = fsedc(ji,jj)                                   !! C  sedimentation flux
                     trc2d(ji,jj,73) = fsedca(ji,jj)                                  !! Ca sedimentation flux
                  endif
                  if (jk.eq.1)  trc2d(ji,jj,74) = qsr(ji,jj)
                  if (jk.eq.1)  trc2d(ji,jj,75) = xpar(ji,jj,jk)
                  !! if (jk.eq.1)  trc2d(ji,jj,75) = real(iters(ji,jj))
                  !! diagnostic fields 76 to 80 calculated below
                  trc2d(ji,jj,81) = trc2d(ji,jj,81) + fprn_ml(ji,jj)           !! mixed layer non-diatom production
                  trc2d(ji,jj,82) = trc2d(ji,jj,82) + fprd_ml(ji,jj)           !! mixed layer     diatom production
# if defined key_gulf_finland
                  if (jk.eq.1)  trc2d(ji,jj,83) = real(ibio_switch)            !! Gulf of Finland check
# else
                  trc2d(ji,jj,83) = ocal_ccd(ji,jj)                            !! calcite CCD depth
# endif
                  trc2d(ji,jj,84) = fccd(ji,jj)                                !! last model level above calcite CCD depth
                  if (jk.eq.1)     trc2d(ji,jj,85) = xFree(ji,jj)              !! surface "free" iron
                  if (jk.eq.i0200) trc2d(ji,jj,86) = xFree(ji,jj)              !! "free" iron at  100 m
                  if (jk.eq.i0200) trc2d(ji,jj,87) = xFree(ji,jj)              !! "free" iron at  200 m
                  if (jk.eq.i0500) trc2d(ji,jj,88) = xFree(ji,jj)              !! "free" iron at  500 m
                  if (jk.eq.i1000) trc2d(ji,jj,89) = xFree(ji,jj)              !! "free" iron at 1000 m
                  !! AXY (27/06/12): extract "euphotic depth"
                  if (jk.eq.1)     trc2d(ji,jj,90) = xze(ji,jj)
                  !! 
# if defined key_roam
                  !! ROAM provisionally has access to a further 20 2D diagnostics
                  if (jk .eq. 1) then
                     trc2d(ji,jj,91)  = trc2d(ji,jj,91)  + wndm(ji,jj)              !! surface wind
                     trc2d(ji,jj,92)  = trc2d(ji,jj,92)  + f_pco2atm(ji,jj)           !! atmospheric pCO2
                     trc2d(ji,jj,93)  = trc2d(ji,jj,93)  + f_ph(ji,jj)                !! ocean pH
                     trc2d(ji,jj,94)  = trc2d(ji,jj,94)  + f_pco2w(ji,jj)             !! ocean pCO2
                     trc2d(ji,jj,95)  = trc2d(ji,jj,95)  + f_h2co3(ji,jj)             !! ocean H2CO3 conc.
                     trc2d(ji,jj,96)  = trc2d(ji,jj,96)  + f_hco3(ji,jj)              !! ocean HCO3 conc.
                     trc2d(ji,jj,97)  = trc2d(ji,jj,97)  + f_co3(ji,jj)               !! ocean CO3 conc.
                     trc2d(ji,jj,98)  = trc2d(ji,jj,98)  + f_co2flux(ji,jj)           !! air-sea CO2 flux
                     trc2d(ji,jj,99)  = trc2d(ji,jj,99)  + f_omcal(ji,jj)      !! ocean omega calcite 
                     trc2d(ji,jj,100) = trc2d(ji,jj,100) + f_omarg(ji,jj)      !! ocean omega aragonite
                     trc2d(ji,jj,101) = trc2d(ji,jj,101) + f_TDIC(ji,jj)              !! ocean TDIC
                     trc2d(ji,jj,102) = trc2d(ji,jj,102) + f_TALK(ji,jj)              !! ocean TALK
                     trc2d(ji,jj,103) = trc2d(ji,jj,103) + f_kw660(ji,jj)             !! surface kw660
                     trc2d(ji,jj,104) = trc2d(ji,jj,104) + f_pp0(ji,jj)               !! surface pressure
                     trc2d(ji,jj,105) = trc2d(ji,jj,105) + f_o2flux(ji,jj)            !! air-sea O2 flux
                     trc2d(ji,jj,106) = trc2d(ji,jj,106) + f_o2sat(ji,jj)             !! ocean O2 saturation
                     trc2d(ji,jj,107) = f2_ccd_cal(ji,jj)                      !! depth calcite CCD
                     trc2d(ji,jj,108) = f2_ccd_arg(ji,jj)                      !! depth aragonite CCD
                  endif
                  if (jk .eq. mbathy(ji,jj)) then
                     trc2d(ji,jj,109) = f3_omcal(ji,jj,jk)                     !! seafloor omega calcite
                     trc2d(ji,jj,110) = f3_omarg(ji,jj,jk)                     !! seafloor omega aragonite
                  endif
                  !! diagnostic fields 111 to 117 calculated below
                  if (jk.eq.i0100) trc2d(ji,jj,118) = ffastca(ji,jj)/MAX(ffastc(ji,jj), rsmall)  !! rain ratio at  100 m
                  if (jk.eq.i0500) trc2d(ji,jj,119) = ffastca(ji,jj)/MAX(ffastc(ji,jj), rsmall)  !! rain ratio at  500 m
                  if (jk.eq.i1000) trc2d(ji,jj,120) = ffastca(ji,jj)/MAX(ffastc(ji,jj), rsmall)  !! rain ratio at 1000 m
                  !! AXY (18/01/12): benthic flux diagnostics
                  if (jk.eq.mbathy(ji,jj)) then
                     trc2d(ji,jj,121) = f_sbenin_n(ji,jj)  + f_fbenin_n(ji,jj)
                     trc2d(ji,jj,122) = f_sbenin_fe(ji,jj) + f_fbenin_fe(ji,jj)
                     trc2d(ji,jj,123) = f_sbenin_c(ji,jj)  + f_fbenin_c(ji,jj)
                     trc2d(ji,jj,124) = f_fbenin_si(ji,jj)
                     trc2d(ji,jj,125) = f_fbenin_ca(ji,jj)
                     trc2d(ji,jj,126) = f_benout_n(ji,jj)
                     trc2d(ji,jj,127) = f_benout_fe(ji,jj)
                     trc2d(ji,jj,128) = f_benout_c(ji,jj)
                     trc2d(ji,jj,129) = f_benout_si(ji,jj)
                     trc2d(ji,jj,130) = f_benout_ca(ji,jj)
                  endif
                  !! diagnostics fields 131 to 135 calculated below
                  trc2d(ji,jj,136) = f_runoff(ji,jj)
                  !! AXY (19/07/12): amended to allow for riverine nutrient addition below surface
                  trc2d(ji,jj,137) = trc2d(ji,jj,137) + (f_riv_loc_n(ji,jj) * fse3t(ji,jj,jk))
                  trc2d(ji,jj,138) = trc2d(ji,jj,138) + (f_riv_loc_si(ji,jj) * fse3t(ji,jj,jk))
                  trc2d(ji,jj,139) = trc2d(ji,jj,139) + (f_riv_loc_c(ji,jj) * fse3t(ji,jj,jk))
                  trc2d(ji,jj,140) = trc2d(ji,jj,140) + (f_riv_loc_alk(ji,jj) * fse3t(ji,jj,jk))
                  trc2d(ji,jj,141) = trc2d(ji,jj,141) + (fslowc(ji,jj)  * fse3t(ji,jj,jk))       !! slow sinking detritus C production
                  if (jk.eq.i0100) trc2d(ji,jj,142) = fslowcflux(ji,jj)        !! slow detritus flux at  100 m
                  if (jk.eq.i0200) trc2d(ji,jj,143) = fslowcflux(ji,jj)        !! slow detritus flux at  200 m
                  if (jk.eq.i0500) trc2d(ji,jj,144) = fslowcflux(ji,jj)        !! slow detritus flux at  500 m
                  if (jk.eq.i1000) trc2d(ji,jj,145) = fslowcflux(ji,jj)        !! slow detritus flux at 1000 m
                  trc2d(ji,jj,146)  = trc2d(ji,jj,146)  + ftot_c(ji,jj)        !! carbon     inventory
                  trc2d(ji,jj,147)  = trc2d(ji,jj,147)  + ftot_a(ji,jj)        !! alkalinity inventory
                  trc2d(ji,jj,148)  = trc2d(ji,jj,148)  + ftot_o2(ji,jj)       !! oxygen     inventory
                  if (jk.eq.mbathy(ji,jj)) then
                     trc2d(ji,jj,149) = f_benout_lyso_ca(ji,jj)
                  endif
                  trc2d(ji,jj,150) = fcomm_resp(ji,jj) * fse3t(ji,jj,jk)                  !! community respiration
		  !!
		  !! AXY (14/02/14): a Valentines Day gift to BASIN - a shedload of new
                  !!                 diagnostics that they'll most likely never need!
                  !!                 (actually, as with all such gifts, I'm giving them
                  !!                 some things I'd like myself!)
                  !! 
                  !! ----------------------------------------------------------------------
                  !! linear losses
                  !! non-diatom
                  trc2d(ji,jj,151) = trc2d(ji,jj,151) + (fdpn2(ji,jj)  * fse3t(ji,jj,jk))
                  !! diatom
                  trc2d(ji,jj,152) = trc2d(ji,jj,152) + (fdpd2(ji,jj)  * fse3t(ji,jj,jk))
                  !! microzooplankton
                  trc2d(ji,jj,153) = trc2d(ji,jj,153) + (fdzmi2(ji,jj) * fse3t(ji,jj,jk))
                  !! mesozooplankton
                  trc2d(ji,jj,154) = trc2d(ji,jj,154) + (fdzme2(ji,jj) * fse3t(ji,jj,jk))
                  !! ----------------------------------------------------------------------
                  !! microzooplankton grazing
                  !! microzooplankton messy -> N
                  trc2d(ji,jj,155) = trc2d(ji,jj,155) + (xphi * (fgmipn(ji,jj) + fgmid(ji,jj)) * fse3t(ji,jj,jk))
                  !! microzooplankton messy -> D
                  trc2d(ji,jj,156) = trc2d(ji,jj,156) + ((1. - xbetan) * finmi(ji,jj) * fse3t(ji,jj,jk))
                  !! microzooplankton messy -> DIC
                  trc2d(ji,jj,157) = trc2d(ji,jj,157) + (xphi * ((xthetapn * fgmipn(ji,jj)) + fgmidc(ji,jj)) * fse3t(ji,jj,jk))
                  !! microzooplankton messy -> Dc
                  trc2d(ji,jj,158) = trc2d(ji,jj,158) + ((1. - xbetac) * ficmi(ji,jj) * fse3t(ji,jj,jk))
                  !! microzooplankton excretion
                  trc2d(ji,jj,159) = trc2d(ji,jj,159) + (fmiexcr(ji,jj) * fse3t(ji,jj,jk))
                  !! microzooplankton respiration
                  trc2d(ji,jj,160) = trc2d(ji,jj,160) + (fmiresp(ji,jj) * fse3t(ji,jj,jk))
                  !! microzooplankton growth
                  trc2d(ji,jj,161) = trc2d(ji,jj,161) + (fmigrow(ji,jj) * fse3t(ji,jj,jk))
                  !! ----------------------------------------------------------------------
                  !! mesozooplankton grazing
                  !! mesozooplankton messy -> N
                  trc2d(ji,jj,162) = trc2d(ji,jj,162) + (xphi * (fgmepn(ji,jj) + fgmepd(ji,jj) + fgmezmi(ji,jj) + fgmed(ji,jj)) * fse3t(ji,jj,jk))
                  !! mesozooplankton messy -> D
                  trc2d(ji,jj,163) = trc2d(ji,jj,163) + ((1. - xbetan) * finme(ji,jj) * fse3t(ji,jj,jk))
                  !! mesozooplankton messy -> DIC
                  trc2d(ji,jj,164) = trc2d(ji,jj,164) + (xphi * ((xthetapn * fgmepn(ji,jj)) + (xthetapd * fgmepd(ji,jj)) + &
                  &                  (xthetazmi * fgmezmi(ji,jj)) + fgmedc(ji,jj)) * fse3t(ji,jj,jk))
                  !! mesozooplankton messy -> Dc
                  trc2d(ji,jj,165) = trc2d(ji,jj,165) + ((1. - xbetac) * ficme(ji,jj) * fse3t(ji,jj,jk))
                  !! mesozooplankton excretion
                  trc2d(ji,jj,166) = trc2d(ji,jj,166) + (fmeexcr(ji,jj) * fse3t(ji,jj,jk))
                  !! mesozooplankton respiration
                  trc2d(ji,jj,167) = trc2d(ji,jj,167) + (fmeresp(ji,jj) * fse3t(ji,jj,jk))
                  !! mesozooplankton growth
                  trc2d(ji,jj,168) = trc2d(ji,jj,168) + (fmegrow(ji,jj) * fse3t(ji,jj,jk))
                  !! ----------------------------------------------------------------------
                  !! miscellaneous
                  trc2d(ji,jj,169) = trc2d(ji,jj,169) + (fddc(ji,jj)    * fse3t(ji,jj,jk)) !! detrital C remineralisation
                  trc2d(ji,jj,170) = trc2d(ji,jj,170) + (fgmidc(ji,jj)  * fse3t(ji,jj,jk)) !! microzoo grazing on detrital carbon
                  trc2d(ji,jj,171) = trc2d(ji,jj,171) + (fgmedc(ji,jj)  * fse3t(ji,jj,jk)) !! mesozoo  grazing on detrital carbon
                  !!
                  !! ----------------------------------------------------------------------
		  !!
		  !! AXY (23/10/14): extract primary production related surface fields to
		  !!                 deal with diel cycle issues; hijacking BASIN 150m
		  !!                 diagnostics to do so (see commented out diagnostics
		  !!                 below this section)
		  !!
                  !! extract relevant BASIN fields at 150m
                  if (jk .eq. i0150) then
                     trc2d(ji,jj,172) = trc2d(ji,jj,4)    !! Pn PP
                     trc2d(ji,jj,173) = trc2d(ji,jj,151)  !! Pn linear loss
                     trc2d(ji,jj,174) = trc2d(ji,jj,5)    !! Pn non-linear loss
                     trc2d(ji,jj,175) = trc2d(ji,jj,11)   !! Pn grazing to Zmi
                     trc2d(ji,jj,176) = trc2d(ji,jj,14)   !! Pn grazing to Zme
                     trc2d(ji,jj,177) = trc2d(ji,jj,6)    !! Pd PP
                     trc2d(ji,jj,178) = trc2d(ji,jj,152)  !! Pd linear loss
                     trc2d(ji,jj,179) = trc2d(ji,jj,7)    !! Pd non-linear loss
                     trc2d(ji,jj,180) = trc2d(ji,jj,15)   !! Pd grazing to Zme
                     trc2d(ji,jj,181) = trc2d(ji,jj,12)   !! Zmi grazing on D
                     trc2d(ji,jj,182) = trc2d(ji,jj,170)  !! Zmi grazing on Dc
                     trc2d(ji,jj,183) = trc2d(ji,jj,155)  !! Zmi messy feeding loss to N
                     trc2d(ji,jj,184) = trc2d(ji,jj,156)  !! Zmi messy feeding loss to D
                     trc2d(ji,jj,185) = trc2d(ji,jj,157)  !! Zmi messy feeding loss to DIC
                     trc2d(ji,jj,186) = trc2d(ji,jj,158)  !! Zmi messy feeding loss to Dc
                     trc2d(ji,jj,187) = trc2d(ji,jj,159)  !! Zmi excretion
                     trc2d(ji,jj,188) = trc2d(ji,jj,160)  !! Zmi respiration
                     trc2d(ji,jj,189) = trc2d(ji,jj,161)  !! Zmi growth
                     trc2d(ji,jj,190) = trc2d(ji,jj,153)  !! Zmi linear loss
                     trc2d(ji,jj,191) = trc2d(ji,jj,13)   !! Zmi non-linear loss
                     trc2d(ji,jj,192) = trc2d(ji,jj,16)   !! Zmi grazing to Zme
                     trc2d(ji,jj,193) = trc2d(ji,jj,17)   !! Zme grazing on D
                     trc2d(ji,jj,194) = trc2d(ji,jj,171)  !! Zme grazing on Dc
                     trc2d(ji,jj,195) = trc2d(ji,jj,162)  !! Zme messy feeding loss to N
                     trc2d(ji,jj,196) = trc2d(ji,jj,163)  !! Zme messy feeding loss to D
                     trc2d(ji,jj,197) = trc2d(ji,jj,164)  !! Zme messy feeding loss to DIC
                     trc2d(ji,jj,198) = trc2d(ji,jj,165)  !! Zme messy feeding loss to Dc
                     trc2d(ji,jj,199) = trc2d(ji,jj,166)  !! Zme excretion
                     trc2d(ji,jj,200) = trc2d(ji,jj,167)  !! Zme respiration
                     trc2d(ji,jj,201) = trc2d(ji,jj,168)  !! Zme growth
                     trc2d(ji,jj,202) = trc2d(ji,jj,154)  !! Zme linear loss
                     trc2d(ji,jj,203) = trc2d(ji,jj,18)   !! Zme non-linear loss
                     trc2d(ji,jj,204) = trc2d(ji,jj,20)   !! Slow detritus production, N
                     trc2d(ji,jj,205) = trc2d(ji,jj,21)   !! Slow detritus remineralisation, N
                     trc2d(ji,jj,206) = trc2d(ji,jj,141)  !! Slow detritus production, C
                     trc2d(ji,jj,207) = trc2d(ji,jj,169)  !! Slow detritus remineralisation, C
                     trc2d(ji,jj,208) = trc2d(ji,jj,43)   !! Fast detritus production, N
                     trc2d(ji,jj,209) = trc2d(ji,jj,21)   !! Fast detritus remineralisation, N
                     trc2d(ji,jj,210) = trc2d(ji,jj,64)   !! Fast detritus production, C
                     trc2d(ji,jj,211) = trc2d(ji,jj,67)   !! Fast detritus remineralisation, C
                     trc2d(ji,jj,212) = trc2d(ji,jj,150)  !! Community respiration
                     trc2d(ji,jj,213) = fslownflux(ji,jj) !! Slow detritus N flux at 150 m
                     trc2d(ji,jj,214) = fslowcflux(ji,jj) !! Slow detritus C flux at 150 m
                     trc2d(ji,jj,215) = ffastn(ji,jj)     !! Fast detritus N flux at 150 m
                     trc2d(ji,jj,216) = ffastc(ji,jj)     !! Fast detritus C flux at 150 m
                  endif
                  !! 
                  !! Jpalm (11-08-2014)
                  !! Add UKESM1 diagnoatics 
                  !!^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
                  if ((jk .eq. 1) .and.( jdms.eq.1)) then
                     trc2d(ji,jj,221) = dms_surf(ji,jj)          !! DMS surface concentration 
                     !! AXY (13/03/15): add in other DMS estimates
                     trc2d(ji,jj,222) = dms_andr(ji,jj)          !! DMS surface concentration 
                     trc2d(ji,jj,223) = dms_simo(ji,jj)          !! DMS surface concentration 
                     trc2d(ji,jj,224) = dms_aran(ji,jj)          !! DMS surface concentration 
                     trc2d(ji,jj,225) = dms_hall(ji,jj)          !! DMS surface concentration 
                  endif
# endif
                  !! other possible future diagnostics include:
                  !!   - integrated tracer values (esp. biological)
                  !!   - mixed layer tracer values
                  !!   - sub-surface chlorophyll maxima (plus depth)
                  !!   - different mixed layer depth criteria (T, sigma, var. sigma)

                  !!----------------------------------------------------------------------
                  !! Prepare 3D diagnostics
                  !!----------------------------------------------------------------------
                  !!
                  trc3d(ji,jj,jk,1)  = ((fprn(ji,jj) + fprd(ji,jj)) * zphn(ji,jj))     !! primary production  
                  trc3d(ji,jj,jk,2)  = fslownflux(ji,jj) + ffastn(ji,jj) !! detrital flux
                  trc3d(ji,jj,jk,3)  = fregen(ji,jj) + (freminn(ji,jj) * fse3t(ji,jj,jk))  !! remineralisation
# if defined key_roam
                  trc3d(ji,jj,jk,4)  = f3_pH(ji,jj,jk)            !! pH
                  trc3d(ji,jj,jk,5)  = f3_omcal(ji,jj,jk)         !! omega calcite
# else
                  trc3d(ji,jj,jk,4)  = ffastsi(ji,jj)             !! fast Si flux
# endif
             ENDIF   ! end of ln_diatrc option
             !! CLOSE wet point IF..THEN loop
            endif
         !! CLOSE horizontal loops
         ENDDO
         ENDDO
         !!
             IF( lk_iomput  .AND.  .NOT.  ln_diatrc  ) THEN

                !!-------------------------------------------------------
                !! 2d specific k level diags
                !!-------------------------------------------------------
                CALL bio_medusa_diag_slice( jk )

              ENDIF
      !! CLOSE vertical loop
      ENDDO

      !!----------------------------------------------------------------------
      !! Final calculations for diagnostics
      !!----------------------------------------------------------------------
      CALL bio_medusa_fin( kt )

# if defined key_trc_diabio
       !! Lateral boundary conditions on trcbio
       DO jn=1,jp_medusa_trd
          CALL lbc_lnk(trbio(:,:,1,jn),'T',1. )
       ENDDO 
# endif

# if defined key_debug_medusa
       IF(lwp) WRITE(numout,*) ' MEDUSA exiting trc_bio_medusa at kt =', kt
       CALL flush(numout)
# endif

   END SUBROUTINE trc_bio_medusa

#else
   !!======================================================================
   !!  Dummy module :                                   No MEDUSA bio-model
   !!======================================================================
CONTAINS
   SUBROUTINE trc_bio_medusa( kt )                   ! Empty routine
      INTEGER, INTENT( in ) ::   kt
      WRITE(*,*) 'trc_bio_medusa: You should not have seen this print! error?', kt
   END SUBROUTINE trc_bio_medusa
#endif 

   !!======================================================================
END MODULE  trcbio_medusa
